﻿  ; Loads the files with synonyms
  ;
  ; V1.0 - 10/MAY/2013
  ; V1.1 - 13/MAY/2013
  ;        o Now exits the procedure if no file is selected.
  ; V1.2 - 14/MAY/2013
  ;        o Fix to window stop responding if we click on it while adding synonyms
  ; V1.3 - 19/MAY/2013
  ;        o Now uses filename string variable, to show it on the window
  ;        o Optimised a little the loading of the synonyms
  ;        o Removes the current synonyms from the gadgetlist
  ;        o Now checks if file exists
  ;        o File buffer of 64kB
  ; V1.4 - 7/JUN/2013
  ;        Now it checks if Thesaurus is in UTF-8 and gives a warning if not
  ; V1.5 - 8/JUN/2013
  ;        <ESC> now aborts loading
  ; V1.6 - 19/JUN/2013
  ;        o Now it works with the optimised "|" in synonyms
  ;        o A HUGE speed gain thanks to Ashley from the PureBasic Forum
  ; V1.7 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)
  ; V1.8 - 26/JUN/2013
  ;        o Changed the non UTF-8 warning to a better one
  ;        o Changed the WindowTimer code to make it work better (faster) in Linux.
  ; V1.9 - 10/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V2.0 - 15/JUL/2013
  ;        o It now uses the variable thesaurus_changed to tell it has been changed
  ; V2.1 - 21/MAR/2014  
  ;        o Resets progress bar when task completes      
  ; V2.2 - 22/APR/2015
  ;        o In the requesters complaining about "UTF-8" it now says "UTF-8 without BOM"
  ; V2.3 - 30/OCT/2015
  ;        o Replaced the "UTF-8 W/O BOM" with "UTF-8 WITHOUT BOM" to make it more understandable.
  ; V2.4 - 11/FEB/2016
  ;        o Thesaurus now uses dynamic arrays
  ; V2.5 - 31/OCT/2017
  ;        o Support for "Recent Files" menu entry
  ; V2.6 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V2.7 - 30/SEP/2019
  ;        o Replaced "UTF-8 WITHOUT BOM" with just "UTF-8". 
  ; V2.8 - 10/FEB/2020
  ;        o Now reads BOM files (3-FEB added) properly without the warning visible on Autocorrect.
  ; V2.9 - 06/JAN/2021
  ;        o Options that use file open show on the status bar the filesize after "OK."
  ; V3.0 - 02/JUL/2021
  ;        o Now the sizes of load/save main files for each tab shows the files label. Ex: "(aff:1 MB)+(dic: 10MB)"  
  ; V3.1 - 22/NOV/2021
  ;        23/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure load_thesaurus(force_file$)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading Thesaurus... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
    
    
    ; Confirm if to load a new Thesaurus if the file in memory has been changed
    If thesaurus_changed=#True
      If MessageRequester("Warning!","The Thesaurus has been changed."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn 0 : EndIf
    EndIf
    
    
    If force_file$=""
      file$=OpenFileRequester("Open Thesaurus", "", "Thesaurus (*.dat)|*.dat", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        ProcedureReturn 0
      EndIf
    Else
      file$=force_file$
    EndIf
    
    
    If IsFileExist(file$,"","File not found.")=#False
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "File not found.")
      ProcedureReturn #False
    EndIf
    
    
    ; Check If Thesaurus is in UTF-8 format
    ReadFile(1,file$)
    ReadStringFormat(1)
    text$=ReadString(1,#PB_UTF8)
    CloseFile(1)
    If LCase(text$)<>"utf-8"
      If MessageRequester("Warning!","Thesaurus is not in "+Chr(34)+"UTF-8-BOM"+Chr(34)+" format:"+#LF$+text$+#LF$+#LF$+"It may lead to corrupt results."+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        ProcedureReturn #False
      EndIf
    EndIf
    
    synonym_disable_undo_redo() ; 8/JUN/2017
    
    ; Get the filesize of the .IDX - 6/JAN/2021
    file_idx$=Left(file$,Len(file$)-4)+".idx"
    OpenFile(2,file_idx$)
      filesize_idx=Lof(2)
    CloseFile(2)
    
    
    thesaurus_changed=#False ; Thesaurus set to unchanged
    If number_of_synonyms>0 : ClearGadgetItems(#PANEL_BOX_SYNONYMS) : EndIf ; To remove the current synonyms from the gadgetlist
    AddWindowTimer(#WINDOW_MAIN,800,800)
    ReadFile(1,File$)
      ReadStringFormat(1)
      number_of_synonyms=0
      Dim synonyms$(0)
      Dim synonyms_selected(0)
      number_of_synonyms_total_selected=0
      UTF_thesaurus$=ReadString(1,#PB_UTF8)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,Lof(1))    
      time=ElapsedMilliseconds()
      time_taken=ElapsedMilliseconds()
      Repeat
        text$=ReadString(1,#PB_UTF8)
        t=Val(StringField(text$,2,"|"))
        If t>#max_number_meanings
          MessageRequester("Warning!","Found synonym with more than "+Str(#max_number_meanings)+" meanings.",#PB_MessageRequester_Warning)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          CloseFile(1)
          RemoveWindowTimer(#WINDOW_MAIN,500)
          ProcedureReturn #False
        EndIf
        synonym$=text$
        For f=1 To t
          synonym$+#LF$+ReadString(1,#PB_UTF8)
        Next f
        
        ReDim synonyms$(number_of_synonyms)
        ReDim synonyms_selected(number_of_synonyms)
        synonyms$(number_of_synonyms)=synonym$     
        number_of_synonyms+1
         
        ; Fix to window stop responding if we click on it while adding synonyms
        event = WindowEvent()
        If Event = #PB_Event_Timer And EventTimer() = 800
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading Thesaurus... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,Loc(1),#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
         
        If ElapsedMilliseconds()>time+800
          update_number_synonyms()
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,Loc(1),#PB_StatusBar_Raised,0,Lof(1))
          time=ElapsedMilliseconds() 
        EndIf
      Until Eof(1)
      filesize_dat=Lof(1)
      CloseFile(1)
      update_number_synonyms()
      RemoveWindowTimer(#WINDOW_MAIN,800)
      thesaurus_synonyms_filename$=file$
      update_filename_synonyms()
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 21/MAR/2014
      
      ; New code to handle the vertical scroll bar
      selected_thesaurus_word=-1
      SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
      SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,0)
      global_update_virtual_vertical_scroll_bar(-1)      
            
      ; 31/OCT/2017 - Support for "Recent Files" menu entry
      create_load_update_recent_files(file$)
      update_recent_files_menu()         
      
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("idx:",filesize_idx,#True)+"+"+ShowFileSizeFormatted("dat:",filesize_dat,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      
  EndProcedure
  
  
  
  
  ; SAVE/SAVE AS
  ; if save=1 and saveas=0 then it saves
  ; if save=0 and saveas=1 then it saves as
  ;
  ; V1.0 - 22/MAY/2013
  ; V1.1 - 25/MAY/2013
  ;        o Finished coding it
  ; V1.2 - <ESC> now aborts saving
  ; V1.3 - 19/JUN/2013
  ;        o Optimised the code by making use of the new "|" tactic on the synonyms
  ; V1.4 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)
  ; V1.5 - 26/JUN/2013
  ;        o Changed the WindowTimer code to make it work better (faster) in Linux.
  ; V1.6 - 11/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V1.7 - 15/JUL/2013
  ;        o It now uses the variable thesaurus_changed to tell it has been changed
  ; V1.8 - 27/MAR/2014  
  ;        o Resets progress bar when task completes   
  ; V1.9 - 7/JUL/2015
  ;        o Saving the Thesaurus now creates an .idx file
  ; V2.0 - 11/FEB/2016
  ;        o Thesaurus now uses dynamic arrays
  ; V2.1 - 11/SEP/2016
  ;        o Now it saves Dictionary + Thesaurus + Autocorrect with #LF$ instead of #CRLF for Linux compatibility
  ; V2.2 - 31/OCT/2017
  ;        o Support for "Recent Files" menu entry
  ; V2.3 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V2.4 - 03/FEB/2020
  ;        o Thesaurus now has a BOM
  ; V2.5 - 02/DEC/2020
  ;        o Options that use file save/extract shown on the status bar show the filesize after "Okay."
  ; V2.6 - 02/JUL/2021
  ;        o Now the sizes of load/save main files for each tab shows the files label. Ex: "(aff:1 MB)+(dic: 10MB)"  
  ; V2.7 - 22/NOV/2021
  ;        23/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure save_saveas_thesaurus(save, saveas)       
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving Thesaurus... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    
    If save=1 : file$=thesaurus_synonyms_filename$ : EndIf
    
    If save=1 And thesaurus_synonyms_filename$="" : saveas=1 : EndIf
    
    If saveas=1
      file$=SaveFileRequester("Save Thesaurus", "", "Thesaurus (*.dat)|*.dat", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf
      If LCase(Right(file$,4))<>".dat" : file$+".dat": EndIf
    EndIf
    
    t=ReadFile(1,file$)
    If t<>0 : CloseFile(1) : EndIf
    If t<>0 And saveas=1
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf      
    EndIf    
    
    DeleteFile(file$)
    
    file_idx$=Left(file$,Len(file$)-4)+".idx" ; 7/JUL/2015
    DeleteFile(file_idx$)
      
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(2,file_idx$) ; 7/JUL/2015
      WriteStringFormat(2,#PB_UTF8)
    OpenFile(1,File$)
      WriteStringFormat(1,#PB_UTF8)
      If UTF_thesaurus$="" : UTF_thesaurus$="UTF-8" : EndIf ;Just in case we started with a blank Thesaurus
      WriteString(1,UTF_thesaurus$+#LF$,#PB_UTF8)   
      WriteString(2,UTF_thesaurus$+#LF$,#PB_UTF8)   
      WriteString(2,Str(number_of_synonyms)+#LF$,#PB_UTF8) 
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_synonyms)     
      time_taken=ElapsedMilliseconds()
      For f=0 To number_of_synonyms-1
        
        ; Write to the .idx
        text$=StringField(synonyms$(f),1,#LF$)
        text$=Left(text$,FindString(text$,"|"))
        text$+Str(Loc(1))
        WriteString(2,text$+#LF$,#PB_UTF8)        
        
        ; Write to the .dat
        text$=StringField(synonyms$(f),1,#LF$)
        t=Val(StringField(text$,2,"|"))
        WriteString(1,text$+#LF$,#PB_UTF8)
              
        For ff=1+1 To t+1
          text$=StringField(synonyms$(f),ff,#LF$)
          WriteString(1,text$+#LF$,#PB_UTF8)   
        Next ff
                
        ; Fix to window stop responding if we click on it while adding synonyms
        event = WindowEvent()
        If Event = #PB_Event_Timer And EventTimer() = 800
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving Thesaurus... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        EndIf
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            CloseFile(2)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
        
        ; Update the number of synonyms and the progress bar
        If ElapsedMilliseconds()>time+800
          update_number_synonyms()
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
          time=ElapsedMilliseconds() 
        EndIf        
      Next f
      
    filesize_dat=Lof(1)
    CloseFile(1)
    filesize_idx=Lof(2)
    CloseFile(2)
    RemoveWindowTimer(#WINDOW_MAIN,800)
    thesaurus_synonyms_filename$=file$
      
    ; Thesaurus set to unchanged
    thesaurus_changed=0
    update_number_synonyms()
    
    ; 31/OCT/2017 - Support for "Recent Files" menu entry
    create_load_update_recent_files(file$)
    update_recent_files_menu()      
    
    update_filename_synonyms()
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("idx:",filesize_idx,#True)+"+"+ShowFileSizeFormatted("dat:",filesize_dat,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
  EndProcedure
  
  
  
  
  ; Extract the Thesaurus into a .txt file
  ;
  ; V1.0 - 18/FEB/2016
  ; V1.1 - 10/APR/2017
  ;        o It is now possible to extract a thesaurus in book format (can be later converted to PDF)
  ; V1.2 - 1/DEC/2017
  ;        o Message Requesters now have a sign.
  ; V1.3 - 15/FEB/2020
  ;        o All file operations support BOM.  
  ; V1.4 - 20/NOV/2020
  ;        o Extracting wordlists now shows the filesize of the target file.
  ; V1.5 - 02/DEC/2021
  ;        o Started showing the time taken in timely operations.
  ;        
  Procedure extract_thesaurus(is_book)       
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Thesaurus... <ESC> To abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    
    
    t$=""
    If is_book=#True : t$=" (Book)" : EndIf
    file$=SaveFileRequester("Extract Thesaurus to file"+t$, "", "Text Documents (*.txt)|*.txt", 0)
    If file$=""
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
    
    
    t=ReadFile(1,file$)
    If t<>0 : CloseFile(1) : EndIf
    If t<>0
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf      
    EndIf    
    
    DeleteFile(file$)
    
      
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(1,File$)
      WriteStringFormat(1,#PB_UTF8)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_synonyms)     
      time_taken=ElapsedMilliseconds()
      For f=0 To number_of_synonyms-1
                
        ; Write to the .txt
        text$=StringField(synonyms$(f),1,#LF$)
        t=Val(StringField(text$,2,"|"))
        WriteString(1,StringField(text$,1,"|")+":"+#LF$,#PB_UTF8)
              
        For ff=1+1 To t+1
          text$=StringField(synonyms$(f),ff,#LF$)
          If is_book=#True
            text$=ReplaceString(text$,"-|","")
            text$=ReplaceString(text$,"|",", ")
            If ff=t+1
              text$+"."
            Else 
              text$+";"
            EndIf  
          EndIf
          WriteString(1,text$+#LF$,#PB_UTF8)   
        Next ff
        WriteString(1,""+#LF$,#PB_UTF8) ;Write a blank line just to separate meanings from the next synonym
                
        ; Fix to window stop responding if we click on it while adding synonyms
        event = WindowEvent()
        If Event = #PB_Event_Timer And EventTimer() = 800
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Autocorrect... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+" ("+StrF(Lof(1)/1024/1024,1)+" MB)"+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
        EndIf
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
        
      Next f
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Autocorrect... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+" ("+StrF(Lof(1)/1024/1024,1)+" MB)"+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      file_size_in_MB.f=Lof(1)/1024/1024
    CloseFile(1)
    RemoveWindowTimer(#WINDOW_MAIN,800)
    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+" ("+StrF(file_size_in_MB.f,1)+" MB)"+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
  EndProcedure
  
  
  
  
  ; To always have the current Thesaurus word highlighted in the listicongadget
  ; V1.0 - 11/FEB/2016
  ;
  Procedure update_vertical_scroll_bar_thesaurus_list_current_item()  
    If selected_thesaurus_word=-1 And GetGadgetState(#PANEL_BOX_SYNONYMS)=-1 : ProcedureReturn #False : EndIf
  
    selected_thesaurus_word=GetGadgetState(#PANEL_BOX_SYNONYMS)+GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)   
    t=GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)
    t1=selected_thesaurus_word-t
    
  EndProcedure
  
  
  
  
  ; Updates the number of synonyms in the thesaurus
  ; Useful in open, add, delete
  ;
  ; V1.0 - 11/MAY/2013
  ; V1.1 - 21/JUN/2013
  ;        o Improved the code thanks to a tip from Bernd (PureBasic forum)
  ; V1.2 - 15/JUL/2013
  ;        o Now places a "*" if the file has been changed
  ; V1.3 - 4/AUG/2015
  ;        o It now shows: "Synonym: [211/588557]"
  ; V1.4 - 11/FEB/2016
  ;        o Thesaurus now uses dynamic array
  ; V1.5 - 25/OCT/2017
  ;        o Formats the total number of items to be better seen.
  ; V1.6 - 20/JUL/2018
  ;        o Now the changes allow a colour asterisk in red
  ; V1.7 - 23/JUL/2018
  ;        o Fix: Asterisks in red no longer flash  
  ; V1.8 - 27/JUL/2018
  ;        28/JUL/2018
  ;        o Fix: Reduced asterisks vanishing if scrolling holding down the arrow keys
  ; V1.9 - 19/SEP/2018
  ;        o Floppy icon on tabs (blue/red)
  ;
  Procedure update_number_synonyms()
    
    ; Black/Red floppy
    If thesaurus_changed=#False And GetGadgetText(#TEXT_PANEL_BOX_SYNONYMS_NUMBER_SYNONYMS_ASTERISK)<>""
      SetGadgetItemImage(#PANEL_MAIN,1,ImageID(71))
    EndIf
    If thesaurus_changed=#True And GetGadgetText(#TEXT_PANEL_BOX_SYNONYMS_NUMBER_SYNONYMS_ASTERISK)<>"*"
      SetGadgetItemImage(#PANEL_MAIN,1,ImageID(72))
    EndIf  
    
    t$=""
    If thesaurus_changed=#True : t$="*" : EndIf
    If selected_thesaurus_word=-1
      t1$="# Synonyms:"+FormatNumber(number_of_synonyms,0,".",number_separator_chr$)
    Else
      t1$="Synonym: ["+FormatNumber(selected_thesaurus_word+1,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_synonyms,0,".",number_separator_chr$)+"]"
    EndIf
    
    If t1$<>GetGadgetText(#TEXT_PANEL_BOX_SYNONYMS_NUMBER_SYNONYMS)
      gadget=#TEXT_PANEL_BOX_SYNONYMS_NUMBER_SYNONYMS
      gadget2=#TEXT_PANEL_BOX_SYNONYMS_NUMBER_SYNONYMS_ASTERISK
      w=GadgetWidth(gadget)
      SetGadgetText(gadget,t1$)
      GadgetFit(gadget)
      If GadgetX(gadget2)<GadgetX(gadget)+GadgetWidth(gadget) Or GadgetX(gadget2)>=GadgetX(gadget)+GadgetWidth(gadget)+4
        ResizeGadget(gadget2,GadgetX(gadget)+GadgetWidth(gadget),GadgetY(gadget),16,16)                        
        GadgetRefresh(gadget2)  
      EndIf
    EndIf     

    If GetGadgetText(#TEXT_PANEL_BOX_SYNONYMS_NUMBER_SYNONYMS_ASTERISK)<>t$
      SetGadgetText(#TEXT_PANEL_BOX_SYNONYMS_NUMBER_SYNONYMS_ASTERISK,t$)
    EndIf
    
  EndProcedure
  
  
  
  
  ; Updates the filename of the synonyms in the thesaurus
  ; Used in open and save as
  ;
  ; V1.0 - 19/MAY/2013
  ; V1.1 - 21/JUN/2013
  ;        o Improved the code thanks to a tip from Bernd (PureBasic forum)
  ; V1.2 - 11/FEB/2016
  ;        o Simplified the code
  ; V1.3 - 25/FEB/2020
  ;        o Fix: Long filenames now fit in the GUI
  ; V1.4 - 26/FEB/2020
  ;        o Cleaned the code that updates filenames in the GUI  
  ; V1.5 - 19/DEC/2021
  ;        o Replaced "Filename:" with "File:"
  ; V1.6 - 01/NOV/2023
  ;        o Replaced "File:" with "💾:"
  ; V1.7 - 26/NOV/2023
  ;        o Replaced the disk emoji plain text with an image
  ; V1.8 - 13/JAN/2024
  ;        14/JAN/2024
  ;        o It now shows a folder emoji with the path of the filename
  ;
  Procedure update_filename_synonyms()
    
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_SYNONYMS_FILENAME,""+GetOnlyFilename(thesaurus_synonyms_filename$,"n/a"),#True)
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_SYNONYMS_FOLDER,""+GetOnlyPath(thesaurus_synonyms_filename$,"n/a"),#True)
    
  EndProcedure


  
  
  ; Delete selected synonym from the thesaurus
  ;
  ; V1.0 - 22/MAY/2013
  ; V1.1 - 15/JUL/2013
  ;        o It now uses the variable thesaurus_changed to tell it has been changed
  ; V1.2 - 14/AUG/2015
  ;        o It now shows the synonym name
  ; V1.3 - 11/FEB/2016
  ;        o Thesaurus now uses dynamic arrays  
  ; V1.4 - 23/DEC/2016
  ;        o Started implementing the multiselection.
  ; V1.5 - 8/JUN/2017
  ;        o undo/redo support  
  ; V1.6 - 6/NOV/2017
  ;        o Now recognises being called from advanced find  
  ; V1.7 - 1/DEC/2017
  ;        o Message Requesters now have a sign. 
  ;
  Procedure delete_synonym(is_called_from_advance_find)
    
    If number_of_synonyms=0 And is_called_from_advance_find=#False
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    t=selected_thesaurus_word
    If number_of_synonyms>0 And t=-1 And is_called_from_advance_find=#False
      MessageRequester("Warning!","No synonym selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    If number_of_synonyms>0 And t<>-1
      text$=StringField(synonyms$(t),1,#LF$)
      If MessageRequester("Warning!","Really delete synonym?"+#LF$+Chr(34)+text$+Chr(34),#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        
        synonym_store_undo(#False) ; 8/JUN/2017
        
        flag=#False 
        
        ; Only one word in the dictionary
        If number_of_synonyms=1 And flag=#False
          ReDim synonyms$(0)
          ReDim synonyms_selected(0)
          synonyms_selected(0)=#False          
          flag=#True
        EndIf
        
        ; The word to delete is the last
        If number_of_synonyms=t+1 And flag=#False
          ReDim synonyms$(ArraySize(synonyms$())-1)
          ReDim synonyms_selected(ArraySize(synonyms_selected())-1)
          flag=#True
        EndIf
        
        ; The word to delete is inside the list, not the first nor the last item
        If flag=#False
          For f=t+1 To number_of_synonyms-1
            synonyms$(f-1)=synonyms$(f)
            synonyms_selected(f-1)=synonyms_selected(f)
          Next f
          ReDim synonyms$(ArraySize(synonyms$())-1)  
          ReDim synonyms_selected(ArraySize(synonyms_selected())-1) 
          flag=#True
        EndIf
        
        number_of_synonyms-1
        update_number_synonyms_selected_count()
        SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
        selected_thesaurus_word=-1 ; Reset the current word selected
        
        thesaurus_changed=#True
        update_number_synonyms()
        global_update_virtual_vertical_scroll_bar(-1)
        
        ProcedureReturn #True
        
      EndIf
    EndIf 
    
    ProcedureReturn #False
    
  EndProcedure
  
  
  
  
  ; ADD/EDIT SYNONYM
  ; If add=1 and edit=0 then it ADDS
  ; If add=0 and edit=1 then it EDITS
  ;
  ; V1.0 - 23/MAY/2013
  ; V1.1 - 24/MAY/2013
  ;        o Focus on synonym if ADD
  ;        o Focus on meanings if EDIT
  ; V1.2 - 29/MAY/2013
  ;        o Fix: If we add meanings, press okay, edit again and remove from the middle, it would keep the meanings after
  ;        o If no synonym is selected it shows a warning window
  ; V1.3 - 5/JUN/2013
  ;        o The window now appears centered on Linux
  ; V1.4 - 18/JUN/2013
  ;        o The synonyms now use "|"
  ; V1.5 - 22/JUN/2013
  ;        o Improved the maximum number of meanings to use also a "|" and to be updated using a procedure
  ; V1.6 - 7/JUL/2013
  ;        o Optimized the code a little, by getting only the text of colunms with the number of meaning
  ;          and not the full range
  ; V1.7 - 15/JUL/2013
  ;        o It now uses the variable thesaurus_changed to tell it has been changed
  ; V1.8 - 11/FEB/2014
  ;        o Ubuntu/Linux users can close the window in its gadget  
  ; V1.9 - 13/FEB/2014
  ;        o Check if <ESC> was pressed  
  ; V2.0 - 29/OCT/2015
  ;        o Increased the width from 640 to 800 to show more content.
  ; V2.1 - 11/FEB/2016
  ;        o Thesaurus now uses dynamic arrays
  ; V2.2 - 16/FEB/2016  
  ;        o Now reds the textbox if invalid spaces
  ; V2.3 - 15/DEC/2016
  ;        o Thesaurus editor now has a larger window.
  ; V2.4 - 25/DEC/2016
  ;        o Started implementing the multiselection.  
  ; V2.5 - 11/MAR/2017
  ;        o Changed the buttons height to 25.   
  ; V2.6 - 8/JUN/2017
  ;        o undo/redo support  
  ; V2.7 - 23/JUL/2017
  ;        o F1 (help) now works in the edit/new items windows  
  ; V2.8 - 1/DEC/2017
  ;        o Message Requesters now have a sign. 
  ; V2.9 - 3/JAN/2018
  ;        o Support for grey tip on EditorGadget  
  ; V3.0 - 14/FEB/2018
  ;        o Now enlarges according to window size definition
  ; V3.1 - 30/MAY/2019
  ;        o GTK3 Ready  
  ; V3.2 - 20/JUN/2019
  ;        o Preferences allow to choose the font for the language.  
  ;
  Procedure add_edit_synonym(add,edit)
    
    If number_of_synonyms=0 And edit=1
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    t=selected_thesaurus_word
    If t=-1 And edit=1
      MessageRequester("Warning!","No synonym selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    t$="Add a synonym" ;add=1
    If edit=1 : t$="Edit a synonym" : EndIf
    If OpenWindow(#WINDOW_ADD_EDIT,0,0,get_extra_width(100)+(800-640)+640-10+(1024-800),get_extra_height(100)+400-10-10+120,t$,#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_ADD_EDIT,get_extra_width(100)+(800-640)+640-10+(1024-800),get_extra_height(100)+400-10-10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)
    
    ; Synonym field
    TextGadget(#TEXT_WINDOW_ADD_EDIT_SYNONYM,0+6,0+6+2+1,80,default_gadget_height_text,"Synonym:")
    t=selected_thesaurus_word
    t$=""
    tt=0 ; # of meanings in synonym
    If t<>-1 And edit=1
      t$=StringField(synonyms$(t),1,#LF$)
      tt=Val(StringField(t$,2,"|")) ; Get the number of meanings for extra speed - 11/FEB/2016
      t$=Left(t$,FindString(t$,"|")-1) ; remove the | from the synonym
    EndIf
    StringGadget(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM,0+6+100-15,0+6,320,default_gadget_height_string,t$)
    If add=1 : SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM) : EndIf
    SetGadgetFont(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM,FontID(#font_for_language_id))
    
    y=GadgetY(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM)+default_gadget_height_string+5
    
    ; Meanings field
    TextGadget(#TEXT_WINDOW_ADD_EDIT_MEANING,0+6,y,80,default_gadget_height_text,"Meanings:")
    EditorGadget(#EDITOR_WINDOW_ADD_EDIT_MEANING,0+6+100-15,y,get_extra_width(100)+(800-640)+600-20-40-20+(1024-800),-10-2+get_extra_height(100)+300+120)
    t$=""
    If t<>-1 And edit=1
      For f=1+1 To tt+1
        a$=StringField(synonyms$(t),f,#LF$)
        If a$<>"" : t$+a$+#LF$ : EndIf
      Next f
    EndIf
    SetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING,t$)
    If edit=1: SetActiveGadget(#EDITOR_WINDOW_ADD_EDIT_MEANING):EndIf
    ; Lines below to set a tip in the gadget if empty, grey - 3/JAN/2018
    If t$=""
      SetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING,"Usage example:"+#LF$+"(noun)|vessel (generic term)|watercraft (generic term)"+#LF$+"(adj)|chromatic (similar term)"+#LF$+"(noun)|gravy boat|gravy holder|sauceboat|dish (generic term)"+#LF$+"(verb)|ride (generic term)"+#LF$+"(adj)|.45-caliber|.45 calibre|.45-calibre|diameter|diam (related term)")
      SetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_MEANING,#PB_Gadget_FrontColor,#Gray)
    EndIf
    SetGadgetFont(#EDITOR_WINDOW_ADD_EDIT_MEANING,FontID(#font_for_language_id))
        
    ;Show current and maximum number of meanings
    t$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING)
    TextGadget(#TEXT_WINDOW_ADD_EDIT_NUMBER_MEANINGS,0+6+100-15+100+200+(1024-800),get_extra_height(100)+300+10+20+10+120,400,default_gadget_height_text,"")
    TextGadget(#TEXT_WINDOW_ADD_EDIT_NUMBER_MEANINGS_TOP_RIGHT,600-200+10+2,0+6+2,100,default_gadget_height_text,"")
    update_number_meanings()
    
    ; Add the OK and CANCEL buttons
    ButtonGadget(#BUTTON_ADD_EDIT_OKAY,0+6+100-15,get_extra_height(100)+300+10+20+10+120,90,default_gadget_height_button,"OK")
    ButtonGadget(#BUTTON_ADD_EDIT_CANCEL,0+6+100-15+100,get_extra_height(100)+300+10+20+10+120,90,default_gadget_height_button,"Cancel")
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_Escape,1000)
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_F1,1004)
    
    ; Wait for the user to press the OK or CANCEL button
    time=ElapsedMilliseconds()
    cancel=#False
    okay=#False
    Repeat
      
      
      ; Change background of word string gadget if a space is found at the start or end, or two spaces in a row (invalid word)  
      t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM)
      If t$<>"" And (Left(t$,1)=" " Or Right(t$,1)=" " Or FindString(t$,"  ")<>0)
        If GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM,#PB_Gadget_BackColor)<>#Red: SetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM,#PB_Gadget_BackColor,#Red) : EndIf
      Else
        If GetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM,#PB_Gadget_BackColor)<>-1: SetGadgetColor(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM,#PB_Gadget_BackColor,-1) : EndIf
      EndIf         
      
      
      event=WaitWindowEvent()
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf      
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf  
      
      
      ; Shortcut for F1 (help)
      If event=#PB_Event_Menu And EventMenu()=1004
        html_help("")
      EndIf               
      
      
      ; Remove/Add the grey tip text from the EditorGadget
      If GetActiveGadget()=#EDITOR_WINDOW_ADD_EDIT_MEANING And GetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_MEANING,#PB_Gadget_FrontColor)=#Gray
        SetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING,"")
        SetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_MEANING,#PB_Gadget_FrontColor,#Black)  
      EndIf        
      If GetActiveGadget()<>#EDITOR_WINDOW_ADD_EDIT_MEANING And GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING)=""
        SetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING,"Usage example:"+#LF$+"(noun)|vessel (generic term)|watercraft (generic term)"+#LF$+"(adj)|chromatic (similar term)"+#LF$+"(noun)|gravy boat|gravy holder|sauceboat|dish (generic term)"+#LF$+"(verb)|ride (generic term)"+#LF$+"(adj)|.45-caliber|.45 calibre|.45-calibre|diameter|diam (related term)")
        SetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_MEANING,#PB_Gadget_FrontColor,#Gray)
      EndIf        
      
      
      
      Select Event 
        Case #PB_Event_Gadget
         Select EventGadget()
           Case #BUTTON_ADD_EDIT_OKAY
             okay=#True
 
             ; For grey tip
             If GetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_MEANING,#PB_Gadget_FrontColor)=#Gray
               SetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING,"")
             EndIf             
             
             t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM)
             If (t$="" Or t$=" ") And okay=1
               MessageRequester("Warning!","No synonym text.",#PB_MessageRequester_Warning)
               SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM)
               okay=#False
             EndIf
             If (CountString(t$,"  ") Or Left(t$,1)=" " Or Right(t$,1)=" ") And okay=#True
               MessageRequester("Warning!","Invalid spaces in synonym.",#PB_MessageRequester_Warning)
               SetActiveGadget(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM)
               okay=#False
             EndIf
             t$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING) 
             If (t$="" Or t$=" ") And okay=#True
               MessageRequester("Warning!","No meanings text.",#PB_MessageRequester_Warning)
               SetActiveGadget(#EDITOR_WINDOW_ADD_EDIT_MEANING)
               okay=#False               
             EndIf
             If CountString(t$,#LF$+#CR$)>0 And okay=#True
               MessageRequester("Warning!","There are blank meanings.",#PB_MessageRequester_Warning)
               SetActiveGadget(#EDITOR_WINDOW_ADD_EDIT_MEANING)
               okay=#False
             EndIf
             If CountString(t$,#LF$)>#max_number_meanings And okay=#True
               MessageRequester("Warning!","Synonym has more than "+Str(#max_number_meanings)+" meanings.",#PB_MessageRequester_Warning)
               SetActiveGadget(#EDITOR_WINDOW_ADD_EDIT_MEANING)
               okay=#False
             EndIf                      
             If Right(t$,1)<>#LF$ And okay=#True
               t$+#LF$
             EndIf
           Case #BUTTON_ADD_EDIT_CANCEL
             cancel=#True
         EndSelect
      EndSelect

      ;Show current and maximum number of meanings (code repeated to update the value)
      If ElapsedMilliseconds()>time+300
        update_number_meanings()        
        time=ElapsedMilliseconds()
      EndIf

    Until okay=#True Or cancel=#True

    
    ; OK button pressed. It will now add or update a synonym and its meanings
    If okay=#True
      t=CountString(GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING),#LF$)
      If Right(GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING),1)<>#LF$ : t+1 : EndIf
      t$=GetGadgetText(#TEXTBOX_WINDOW_ADD_EDIT_SYNONYM)+"|"+Str(t)+#LF$
      a$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING)
      a$=RemoveString(a$,#CR$)
      t$+a$
      If Right(t$,1)=#LF$ : t$=Left(t$,Len(t$)-1) : EndIf

      If add=#True
        synonym_store_undo(#False) ; 8/JUN/2017
        ReDim synonyms$(number_of_synonyms)      
        ReDim synonyms_selected(number_of_synonyms)   
        synonyms$(number_of_synonyms)=t$
        number_of_synonyms+1
        SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
        update_number_synonyms()
        global_update_virtual_vertical_scroll_bar(-1)
      EndIf
      If edit=#True
        synonym_store_undo(#False) ; 8/JUN/2017
        t=selected_thesaurus_word         
        synonyms$(t)=t$
        global_update_virtual_vertical_scroll_bar(-1)
      EndIf
    EndIf
    
    ; OK pressed and change the thesaurus_changed variable
    If okay=#True
      thesaurus_changed=#True
      update_number_synonyms()
    EndIf
    
    update_vertical_scroll_bar_thesaurus_current_item()  
    
    ; Close the Add/Edit window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_ADD_EDIT)    

  EndProcedure
  
  
  
  
  ; Update number of meanings in ADD/EDIT synonyms window
  ;
  ; V1.0 - 22/JUN/2013
  ;        o Coded the procedure
  ; V1.1 - 28/SEP/2016
  ;        o Support for Mac OS line endings
  ; V1.2 - 25/OCT/2017
  ;        o Formats the total number of items to be better seen.  
  ; V1.3 - 3/JAN/2018
  ;        o Support for grey tip on EditorGadget
  ;
  Procedure update_number_meanings()
    t=0
    t$=GetGadgetText(#EDITOR_WINDOW_ADD_EDIT_MEANING)
    If GetGadgetColor(#EDITOR_WINDOW_ADD_EDIT_MEANING,#PB_Gadget_FrontColor)=#Gray : t$="" : EndIf    
    If Len(t$)<>0
      t=CountString(t$,#LF$)
      t1=CountString(t$,#CR$)
      If t=0 And t1<>0 : t=t1 : EndIf
    EndIf
    SetGadgetText(#TEXT_WINDOW_ADD_EDIT_NUMBER_MEANINGS,"# Meanings:"+FormatNumber(t,0,".",number_separator_chr$)+"/"+FormatNumber(#max_number_meanings,0,".",number_separator_chr$))
    SetGadgetText(#TEXT_WINDOW_ADD_EDIT_NUMBER_MEANINGS_TOP_RIGHT,"|"+FormatNumber(t,0,".",number_separator_chr$))
  EndProcedure
  
  
  
  
  ; Sorts the list of synonyms
  ;
  ; V1.0 - 22/MAY/2013
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 24/MAY/2013
  ;        o Coded the complete procedure
  ; V1.2 - 27/MAY/2013
  ;        o Sort is now non case sensitive
  ; V1.3 - 12/JUN/2013
  ;        o Now pressing <ESC> aborts copying from the listbox to the array
  ; V1.4 - 19/JUN/2013
  ;        o A HUGE speed gain thanks to Ashley from the PureBasic Forum
  ; V1.5 - 22/JUN/2013
  ;        o Removed the code that fills the entire progress bar at the end of the loop
  ;          and also its delay(500)  
  ; V1.6 - 26/JUN/2013
  ;        o Changed the WindowTimer code to make it work better (faster) in Linux.
  ; V1.7 - 12/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ; V1.8 - 16/JUL/2013
  ;        o It now uses the variable thesaurus_changed to tell it has been changed
  ; V1.9 - 21/MAR/2014  
  ;        o Resets progress bar when task completes  
  ; V2.0 - 11/FEB/2016
  ;        o Thesaurus now uses dynamic arrays  
  ; V2.1 - 22/AUG/2016
  ;        o Fix: Sorting synonyms naturally by replacing | with chr(9) and after sorting with |
  ; V2.2 - 16/SEP/2016
  ;        o Now it sorts naturally the synonyms
  ; V2.3 - 26/SEP/2016
  ;        o Reverted back to the 1st natural sorting: | with chr(9), sort, chr(9) with |
  ; V2.4 - 23/DEC/2016
  ;        o Started implementing the multiselection. 
  ; V2.5 - 08/JUN/2017
  ;        o undo/redo support  
  ; V2.6 - 01/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V2.7 - 13/NOV/2019
  ;        o Support for Latin sorting
  ; V2.8 - 02/DEC/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure sort_synonyms()
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting Thesaurus...")
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
      ProcedureReturn #False
    EndIf
    
    synonym_store_undo(#True) ; 8/JUN/2017
    
    ; Show the progress bar
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_synonyms*3) 
    
    ; Thesaurus set to changed
    selected_thesaurus_word=-1 ; Reset the current word selected
    thesaurus_changed=1
    update_number_synonyms()
    
    
    time_taken=ElapsedMilliseconds()
    AddWindowTimer(#WINDOW_MAIN,800,800)
    
    
    ; Remove accents from entries
    ReDim sort_arrays_in_latin(number_of_synonyms-1)
    For f=0 To number_of_synonyms-1
      
      sort_arrays_in_latin(f)\in_latin=RemoveAccentsString(synonyms$(f))
      sort_arrays_in_latin(f)\in_ascii=synonyms$(f)
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f+1,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing accents... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f
    
    
    ; Sort the array and delete the list 
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting... <ESC> to abort. Found:"+FormatNumber(number_of_synonyms,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    For f=0 To number_of_synonyms-1
      ReplaceString(sort_arrays_in_latin(f)\in_latin,"|",Chr(9),#PB_String_InPlace,1,1)
    Next f    
    SortStructuredArray(sort_arrays_in_latin(),#PB_Sort_Ascending,OffsetOf(sort_arrays_in_latin\in_latin.s),TypeOf(sort_arrays_in_latin\in_latin.s))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,number_of_synonyms*2,#PB_StatusBar_Raised)
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting... <ESC> to abort. Found:"+FormatNumber(number_of_synonyms,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
    
    ; Populate the list with the sorted words
    For f=0 To number_of_synonyms-1
      
      synonyms$(f)=sort_arrays_in_latin(f)\in_ascii
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,(number_of_synonyms*2)+f+1,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating with accents... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f    
    
    
    ; Populate the ListIconGadget
    ClearGadgetItems(#PANEL_BOX_SYNONYMS)       
    SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,0)
    global_update_virtual_vertical_scroll_bar(-1)
    
    
    ; Clear progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,1) ; Reset progress bar - 21/MAR/2014
    
    ; No selected items
    update_number_synonyms_selected_none()
        
    
  EndProcedure
  
  
  
  
  ; Erases the list of synonyms
  ;
  ; V1.0 - 22/MAY/2013
  ; V1.1 - 19/MAY/2013
  ;        o Now set the variable of encoding to "UTF-8".
  ; V1.2 - 14/JUL/2013
  ;        o It now makes use of the status bar suggested by Ashley Scott
  ;        o After a purge, the filename is reset.
  ; V1.3 - 17/JUL/2013
  ;        o It now uses the variable thesaurus_changed to tell it has been changed
  ;        o Checks If there have been changes, And ask before purging
  ; V1.4 - 21/MAR/2014  
  ;        o Resets progress bar when task completes
  ; V1.5 - 11/FEB/2016
  ;        o Thesaurus now uses dynamic arrays  
  ; V1.6 - 23/DEC/2016
  ;        o Started implementing the multiselection.
  ; V1.7 - 8/JUN/2017
  ;        o undo/redo support
  ; V1.8 - 1/DEC/2017
  ;        o Message Requesters now have a sign. 
  ;
  Procedure erase_synonyms()
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf
    
    ; Confirm if to erase the Thesaurus if the file in memory has been changed
    If thesaurus_changed=#True
      If MessageRequester("Warning!","The Thesaurus has been changed."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn #False : EndIf
    EndIf    
    
    If number_of_synonyms>0
      If MessageRequester("Warning!","Really delete all synonyms?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        
        synonym_store_undo(#True) ; 8/JUN/2017
        
        FUCKTIME=ElapsedMilliseconds()
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Erasing...")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        number_of_synonyms=0
        ClearGadgetItems(#PANEL_BOX_SYNONYMS)
        Dim synonyms$(0)
        Dim synonyms_selected(0)
        number_of_synonyms_total_selected=0        
        UTF_thesaurus$="UTF-8"
        update_number_synonyms()
        thesaurus_synonyms_filename$=""
        update_filename_synonyms()
        
 
        selected_thesaurus_word=-1 ; Reset the current word selected
        thesaurus_changed=#False ; Dictionary set to unchanged
        update_number_synonyms()
        global_update_virtual_vertical_scroll_bar(-1)
        Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
       
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,100,#PB_StatusBar_Raised,0,100)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,100) ; Reset progress bar - 21/MAR/2014
      EndIf
    EndIf
    
  EndProcedure
  
  
  
  
; Fix spaces in Thesaurus
;
; V1.0 - 15/FEB/2016
; V1.1 - 8/JUN/2017
;        o undo/redo support  
; V1.2 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.3 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure fix_spaces_thesaurus()
    Debug "Pressed FIX SPACES THESAURUS"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fix Invalid Spaces... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    synonym_store_undo(#True) ; 8/JUN/2017
    
    ; Processing wordlist - STEP 1 of 2
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_synonyms)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
      
    
    word_fixes=0
    For f=0 To number_of_synonyms-1
      text$=synonyms$(f)
      
      Repeat
        text$=ReplaceString(text$,"  "," ")
        text$=ReplaceString(text$,"| ","|")
        text$=ReplaceString(text$," |","|")
        text$=ReplaceString(text$," "+#LF$,#LF$)
        text$=ReplaceString(text$,#LF$+" ",#LF$)
        If Right(text$,1)=" " : text$=Left(text$,Len(text$)-1) : EndIf
      Until FindString(text$,"  ")=0 And FindString(text$,"| ")=0 And FindString(text$," |")=0 And FindString(text$," "+#LF$)=0 And FindString(text$,#LF$+" ")=0 And Right(text$,1)<>" "
      
      ; Abort if an empty word/meaning is found
      original_text$=StringField(synonyms$(f),1,#LF$)
      t=Val(StringField(original_text$,2,"|"))      
      For ff=1 To t+1
        If StringField(text$,ff,#LF$)="" Or StringField(text$,ff,#LF$)="|" Or StringField(text$,ff,#LF$)="-|" Or Left(StringField(text$,1,#LF$),1)="|"
          Debug "NULL WORD FOUND!"
          t$="Meaning:"+Str(ff-1)
          If ff=1 : t$="Synonym": EndIf
          
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Null word at Line:"+FormatNumber(f+1,0,".",number_separator_chr$)+" "+t$+" Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False        
        EndIf
      Next ff
      
      
      ; Increment fixes
      If text$<>synonyms$(f)
        word_fixes+1
      EndIf      
      
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Wordlist 1/2..."+FormatNumber(f,0,".",number_separator_chr$)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    
    ; No fixes found
    If word_fixes=0
      MessageRequester("Warning!","No spaces to fix found!",#PB_MessageRequester_Warning)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR,0,"OK.")
      RemoveWindowTimer(#WINDOW_MAIN,800)
      ProcedureReturn #False      
    EndIf
    
    
    
    ; Fixing spaces - STEP 2 of 2    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixing spaces 2/2...0 <ESC> To abort.")
    
    word_fixes=0
    For f=0 To number_of_synonyms-1
      text$=synonyms$(f)
      
      Repeat
        text$=ReplaceString(text$,"  "," ")
        text$=ReplaceString(text$,"| ","|")
        text$=ReplaceString(text$," |","|")
        text$=ReplaceString(text$," "+#LF$,#LF$)
        text$=ReplaceString(text$,#LF$+" ",#LF$)
        If Right(text$,1)=" " : text$=Left(text$,Len(text$)-1) : EndIf
      Until FindString(text$,"  ")=0 And FindString(text$,"| ")=0 And FindString(text$," |")=0 And FindString(text$," "+#LF$)=0 And FindString(text$,#LF$+" ")=0 And Right(text$,1)<>" "
      
      ; Increment fixes + fix directly in the array
      If text$<>synonyms$(f)
        word_fixes+1
        thesaurus_changed=#True
        synonyms$(f)=text$
      EndIf

      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixing spaces 2/2..."+FormatNumber(f,0,".",number_separator_chr$)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          global_update_virtual_vertical_scroll_bar(-1)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f    
   
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixed words:"+FormatNumber(word_fixes,0,".",number_separator_chr$))  
    global_update_virtual_vertical_scroll_bar(-1)    
    
EndProcedure




; Sort meanings of synonyms
;
; V1.0 - 16/FEB/2016
; V1.1 - 16/FEB/2016
;        o Coded also an option just to unduplicate meanings.
; V1.2 - 14/DEC/2016
;        o Warning for operation involving sorting internal synonyms.
; V1.3 - 8/JUN/2017
;        o undo/redo support  
; V1.4 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure sort_unduplicate_meanings_thesaurus(sort_meanings, unduplicate_meanings)
  
    Debug "Pressed SORT or UNDUPLICATE MEANINGS THESAURUS"
    
    t$="Sort"
    If unduplicate_meanings=#True : t$="Unduplicate" : EndIf
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, t$+" meanings... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    ; Sort simple meanings (one item per line) Warning
    If sort_meanings=#True
      If MessageRequester("Warning!","This changes the sequence of meanings in the synonyms."+#LF$+"If we have the word 'happy' with 3 meanings:"+#LF$+"'contented'"+#LF$+"'satisfied'"+#LF$+"'carefree'"+#LF$+#LF$+"'carefree' will move to the top."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        ProcedureReturn #False
      EndIf
    EndIf
    
    synonym_store_undo(#True) ; 8/JUN/2017
    
    ; Sort/Unduplicate the meanings
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_synonyms)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
      
    unduplicate_meanings_counter=0 ; to store the number of unduplicated synonyms
    
    For f=0 To number_of_synonyms-1
      
      full_synonym_meanings$=synonyms$(f)
      original_text$=StringField(synonyms$(f),1,#LF$)
      t=Val(StringField(original_text$,2,"|"))      
      Dim sort_thesaurus_meanings$(0)
      Dim buffer_thesaurus_meanings$(0)
      For ff=1+1 To t+1
        ReDim sort_thesaurus_meanings$(ff-2)
        sort_thesaurus_meanings$(ff-2)=StringField(full_synonym_meanings$,ff,#LF$)
        ReDim buffer_thesaurus_meanings$(ff-2)
        buffer_thesaurus_meanings$(ff-2)=StringField(full_synonym_meanings$,ff,#LF$) 
      Next ff
      
      
      ; Sort the meanings in the sorting array only
      SortArray(sort_thesaurus_meanings$(),#PB_Sort_Ascending|#PB_Sort_NoCase)
      
      
      ; Search for duplicates and make them = ""
      counter_found=0
      If unduplicate_meanings=#True
;         counter_found=0
        For ff=0 To t-1-1
          If sort_thesaurus_meanings$(ff)=sort_thesaurus_meanings$(ff+1)
            counter_found+1
;             sort_thesaurus_meanings$(ff)=""
            If unduplicate_meanings=#True
              For fff=0 To t-1
                If buffer_thesaurus_meanings$(fff)=sort_thesaurus_meanings$(ff)
                  buffer_thesaurus_meanings$(fff)=""
                  Break
                EndIf
              Next fff
            EndIf
          EndIf
        Next ff
      EndIf
      
      
      ; Transfer the meanings back to the original array
      original_text$=StringField(original_text$,1,"|")+"|"+Str(t-counter_found)
      For ff=0 To t-1
        ; Sorted
        If sort_thesaurus_meanings$(ff)<>"" And sort_meanings=#True
          original_text$+#LF$+sort_thesaurus_meanings$(ff)
        EndIf
        
        ; Unduplicated
        If buffer_thesaurus_meanings$(ff)<>"" And unduplicate_meanings=#True
          original_text$+#LF$+buffer_thesaurus_meanings$(ff)
        EndIf        
        
      Next ff
      If synonyms$(f)<>original_text$
        synonyms$(f)=original_text$
        thesaurus_changed=#True ; Thesaurus changed only if there were changes
        If unduplicate_meanings=#True : unduplicate_meanings_counter+1 : EndIf ; Store the number of unduplicated meanings
;         Debug "F:"+Str(f)+" S:"+synonyms$(f)+" O:"+original_text$
      EndIf
      
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        t$="Sort"
        If unduplicate_meanings=#True : t$="Unduplicate" : EndIf        
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, t$+" meanings..."+Str(f)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          global_update_virtual_vertical_scroll_bar(-1)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    t$="OK."
    If unduplicate_meanings=#True : t$="Unduplicated synonyms meanings:"+Str(unduplicate_meanings_counter) : EndIf
    StatusBarText(#WINDOW_MAIN_STATUS_BAR,0,t$)
    global_update_virtual_vertical_scroll_bar(-1)
    
EndProcedure



  
  ; To always have the current synonym highlighted in the listicongadget
  ;
  ; V1.0 - 11/FEB/2016
  ;
  Procedure update_vertical_scroll_bar_thesaurus_current_item()  
    If selected_thesaurus_word=-1 And GetGadgetState(#PANEL_BOX_SYNONYMS)=-1 : ProcedureReturn #False : EndIf
  
    selected_thesaurus_word=GetGadgetState(#PANEL_BOX_SYNONYMS)+GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)   
    t=GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)
    t1=selected_thesaurus_word-t
    
  EndProcedure
  
  
  

; This procedure will remove the following symbols from the synonyms
; 1) -|
; 2) |
; 3) (...)
;
; V1.0 - 13/AUG/2015
; V1.1 - 14/AUG/2015
;        o Removed the space in -| and (...) but kept on | and added a better description.
; V1.2 - 12/FEB/2016
;        o Thesaurus now uses dynamic arrays
;        o Now checks If it produces a null value (no characters)
; V1.3 - 11/MAR/2017
;        o Changed the buttons height to 25.  
; V1.4 - 8/JUN/2017
;        o undo/redo support  
; V1.5 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.6 - 01/DEC/2017
;        o Message Requesters now have a sign. 
; V1.7 - 09/JUN/2019
;        o GTK3 Ready
;
Procedure cleanup_symbols_synonyms()
  
    Debug "Pressed CLEAN UP SYMBOLS SYNONYMS"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Clean up. 1/3: Extracting... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
      ProcedureReturn 0
    EndIf 

  
    ; Variable to arrange GUI
    t=-30
  
    ; Open window
    If OpenWindow(#WINDOW_CLEANUP,0,0,320+100-30-20-20+20-20-20,200-40-50-20-20+60-20+25+25+t+10+10,"Clean up symbols",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    DisableWindow(#WINDOW_MAIN,#True)
    
    ; Display checkboxes
    CheckBoxGadget(#CHECKBOX_CHR1_WINDOW_CLEANUP,10,t+200-40-50-20-20-20+2+2-5,90+180,default_gadget_height_checkbox,"-|       ; Replace with NOTHING")
    SetGadgetState(#CHECKBOX_CHR1_WINDOW_CLEANUP,cleanup_symbols_chr1)    
    CheckBoxGadget(#CHECKBOX_CHR2_WINDOW_CLEANUP,10,-2+1+t+200-40-50-20-20-20+25-5,90+180,default_gadget_height_checkbox,"|        ; Replace with a SPACE")
    SetGadgetState(#CHECKBOX_CHR2_WINDOW_CLEANUP,cleanup_symbols_chr2)
    CheckBoxGadget(#CHECKBOX_CHR3_WINDOW_CLEANUP,10,-2-1+2+t+200-40-50-20-20+20-20+25-5,90+180,default_gadget_height_checkbox,"(...)    ; Replace with NOTHING")   
    SetGadgetState(#CHECKBOX_CHR3_WINDOW_CLEANUP,cleanup_symbols_chr3)
    
    ; Buttons OK/CANCEL
    ButtonGadget(#WINDOW_CLEANUP_OKAY,320/3-20-20,t+200-40-50-20-20+60-20+25,90,default_gadget_height_button,"OK")
    ButtonGadget(#WINDOW_CLEANUP_CANCEL,320/3+100-20-20,t+200-40-50-20-20+60-20+25,90,default_gadget_height_button,"Cancel")
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_CLEANUP,#PB_Shortcut_Escape,1000)
    AddKeyboardShortcut(#WINDOW_CLEANUP,#PB_Shortcut_Return,1003) 
    
    cancel=#False
    Repeat
            
      event=WaitWindowEvent()
      
      ;Change the settings of the checkboxes and find text box
      t=GetGadgetState(#CHECKBOX_CHR1_WINDOW_CLEANUP)
      If cleanup_symbols_chr1<>t : cleanup_symbols_chr1=t : EndIf
      t=GetGadgetState(#CHECKBOX_CHR2_WINDOW_CLEANUP)
      If cleanup_symbols_chr2<>t : cleanup_symbols_chr2=t : EndIf
      t=GetGadgetState(#CHECKBOX_CHR3_WINDOW_CLEANUP)
      If cleanup_symbols_chr3<>t : cleanup_symbols_chr3=t : EndIf
     
   
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf
     

      Select event 
        Case #PB_Event_Gadget
          Select EventGadget()
                                 
           Case #WINDOW_CLEANUP_OKAY
             Debug "Pressed OK"             
             
             ; Close the window "Clean Up"
             CloseWindow(#WINDOW_CLEANUP)            
             
             synonym_store_undo(#True) ; 8/JUN/2017              
             
             ; Step 1 of 3 (extracting synonyms from listicongadget to array)
             AddWindowTimer(#WINDOW_MAIN,800,800)
             StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_synonyms)     
             time=ElapsedMilliseconds()
             FUCKTIME=ElapsedMilliseconds()
             total_number_of_processed_synonyms=0
             Dim total_number_of_processed_synonyms_array$(0)
    
    
             For f=0 To number_of_synonyms-1
      
                ; Get a line of synonyms
                text$=synonyms$(f)                 
            
                ReDim total_number_of_processed_synonyms_array$(total_number_of_processed_synonyms)
                total_number_of_processed_synonyms_array$(total_number_of_processed_synonyms)=text$ 
                total_number_of_processed_synonyms+1
                      
                ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
                event = WindowEvent()
                If event=#PB_Event_Timer And EventTimer()=800
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Clean up. 1/3: Extracting..."+FormatNumber(total_number_of_processed_synonyms,0,".",number_separator_chr$)+" <ESC> to abort.")
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
                EndIf 
        
                ; <ESC> pressed to abort
                If EventMenu()=1000
                  If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
                    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
                    RemoveWindowTimer(#WINDOW_MAIN,800)
                    
                    ; Activate the main window
                    DisableWindow(#WINDOW_MAIN,#False)
                    
                    ProcedureReturn #False
                  EndIf
                EndIf
        
             Next f
    
   
    
    
             ; Step 2 of 3 (Remove symbols)  
    
             ; Progress bar
             StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Remove symbols. 2/3: Comparing... <ESC> to abort.")    
             StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_number_of_processed_synonyms)     
        
             
             occurrences=0 ;0=No symbol removed   >0 number of symbols removed, to populate the listicongadget if >0
             For f=0 To total_number_of_processed_synonyms-1
               
                new_text$=StringField(total_number_of_processed_synonyms_array$(f),1,#LF$)
                t=Val(StringField(new_text$,2,"|"))
                
                For ff=1+1 To t+1
                  
                  text$=StringField(total_number_of_processed_synonyms_array$(f),ff,#LF$) ; Get each meaning                  
                
                  ; Remove -|
                  If cleanup_symbols_chr1=#True
                    text$=ReplaceString(text$,"-|","") ; Replace with NOTHING
                    occurrences=1
                  EndIf
                
                  ; Remove |
                  If cleanup_symbols_chr2=#True
                    text$=ReplaceString(Mid(text$,pos_chr10),"|"," ") ; Replace with a SPACE
                    occurrences=1
                  EndIf
                
                
                  ; Remove (...)
                  If cleanup_symbols_chr3=#True
                    If CountString(text$,"(")=CountString(text$,")")
                      Repeat
                      
                        flag=#False ;If it is set to #True, it means that a ( or ) has been found                    
                        t1=FindString(text$,"(") ; Find ( position
                        t2=FindString(text$,")") ; Find ) position
                        If t1<t2                      
                          text$=Left(text$,t1-1)+Mid(text$,t2+1) ; Replace with NOTHING
                          flag=#True
                        EndIf

                      Until flag=#False ;It repeats until no ( or ) has been found
                      occurrences=1
                    EndIf
                  EndIf
                  
                
                  ; Check if Null Meaning
                  If text$=""
                    
                    Debug "NULL! INVALID MEANING!"
                    
                    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
                    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Null meaning at Line:"+FormatNumber(f+1,0,".",number_separator_chr$)+" Meaning:"+Str(ff-1)+"! Aborted.")
                    RemoveWindowTimer(#WINDOW_MAIN,800)
                    
                    ; Set the line focus to the invalid line
                    selected_thesaurus_word=f
                    global_update_virtual_vertical_scroll_bar(-1)            
                    
                    ; Activate the main window
                    DisableWindow(#WINDOW_MAIN,#False)

                    ProcedureReturn #False                    
                    
                  EndIf
                  
                  new_text$+#LF$+text$
                  
                Next ff
                
                
                ; Copy back the synonym into the array
                total_number_of_processed_synonyms_array$(f)=new_text$
                
      
                ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
                event = WindowEvent()
                If event=#PB_Event_Timer And EventTimer()=800
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Remove symbols. 2/3: Comparing..."+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(total_number_of_processed_synonyms,0,".",number_separator_chr$)+" <ESC> to abort.")    
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
                EndIf 
                
                
                ; <ESC> pressed to abort
                If EventMenu()=1000
                  If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
                    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
                    RemoveWindowTimer(#WINDOW_MAIN,800)
                    
                    ; Activate the main window
                    DisableWindow(#WINDOW_MAIN,#False)

                    ProcedureReturn #False
                  EndIf
                EndIf      
                
             Next f
             
             
             
      
            ; Step 3 of 3 (populating synonyms which means copying them back to the listicongadget), only if occurences>0

    
            ; Delete the list if occurences>0
            If occurrences>0
              ClearGadgetItems(#PANEL_BOX_SYNONYMS)
            EndIf
    
            ; Progress bar
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Remove symbols. 3/3: Populating... 0/"+FormatNumber(total_number_of_processed_synonyms,0,".",number_separator_chr$))
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_number_of_processed_synonyms)


            ; copy from array back to list
            If occurrences>0
              AddWindowTimer(#WINDOW_MAIN,800,800)
              time=ElapsedMilliseconds()

              For f=0 To total_number_of_processed_synonyms-1
                t$=total_number_of_processed_synonyms_array$(f)
                synonyms$(f)=t$
      
                ; Fix to window stop responding if we click on it while adding synonyms
                event = WindowEvent()
                If Event = #PB_Event_Timer And EventTimer() = 800 : EndIf     
      
                If ElapsedMilliseconds()>time+800
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Remove symbols. 3/3: Populating..."+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(total_number_of_processed_synonyms,0,".",number_separator_chr$))            
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0+f,#PB_StatusBar_Raised,0,total_number_of_processed_synonyms)
                  time=ElapsedMilliseconds() 
                EndIf
      
              Next f    
              RemoveWindowTimer(#WINDOW_MAIN,800)
            EndIf
            
            Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)            
            
            ; Reset status bar and display "OK." in it
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
            If occurrences>0 : thesaurus_changed=#True : EndIf ;Thesaurus changed            

            
            ; Refresh the ListIconGadget     
            global_update_virtual_vertical_scroll_bar(-1) 
            
            ; Activate the main window
            DisableWindow(#WINDOW_MAIN,#False)
            
            ProcedureReturn       
    
    
             
           Case #WINDOW_CLEANUP_CANCEL

            ; Reset status bar and display "OK." in it
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
            
            Debug "Pressed CANCEL"
            cancel=#True
                          

         EndSelect
      EndSelect
    Until cancel=#True
    
    
    ; Close the Clean up window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_CLEANUP)
    
EndProcedure
  



; To find/merge duplicates in the synonyms
; V1.0 -  6/AUG/2015
; V1.1 -  7/AUG/2015
;        o If "Show Duplicates" finds nothing, it opens a requester saying so.
; V1.2 - 10/AUG/2015
;        o Improved the looking for duplicates, now with a preview area, plus exporting also with extra information.
; V1.3 - 11/AUG/2015
;        o Also coded the merging and combine
; V1.4 - 12/AUG/2015
;        o Fixed the merging and combine.
; V1.5 - 14/AUG/2015
;        o Fix: The EditorGadget preview is now editable
; V1.6 - 14/AUG/2015
;        o Cleared the local listicongadget in case one item remains there.   
; V1.7 - 21/JAN/2016
;        o "Display" button now disabled if nothing is selected in the ListIconGadget
; V1.8 - 12/FEB/2016
;        o Thesaurus now uses dynamic arrays
; V1.9 - 25/DEC/2016
;        o Started implementing the multiselection.
; V2.0 - 11/MAR/2017
;        o Changed the buttons height to 25. 
; V2.1 - 8/JUN/2017
;        o undo/redo support  
; V2.2 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V2.3 - 1/DEC/2017
;        o Message Requesters now have a sign.
; V2.4 - 10/JUN/2019
;        o GTK3 Ready  
; V2.5 - 20/JUN/2019
;        o Preferences allow to choose the font for the language.
; V2.6 - 15/FEB/2020
;        o All file operations support BOM.
; V2.7 - 21/FEB/2020
;        o All remaining export operations now use UNIX #LF$
;
Procedure show_merge_duplicates_synonyms()
  
    Debug "Pressed SHOW DUPLICATES SYNONYMS"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
      ProcedureReturn 0
    EndIf 
      
    
    ; Synonyms=1 => No duplicates found, exit the Procedure - 11/AUG/2015
    If number_of_synonyms=1
      MessageRequester("Warning!","No duplicates found.",#PB_MessageRequester_Warning)
      ProcedureReturn
    EndIf    
    
    synonym_store_undo(#True) ; 8/JUN/2017
    
    ; Step 1 of 2 (extracting synonyms from listicongadget to array)
    AddWindowTimer(#WINDOW_MAIN,800,800)
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_synonyms)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    total_number_of_processed_synonyms=0
    Dim total_number_of_processed_synonyms_array$(0)
    Dim total_number_of_original_synonyms_array$(0)
    
    
    For f=0 To number_of_synonyms-1
      
      ; Get a line of synonyms
      text$=synonyms$(f)
      t$=text$ ; To use in the original array now used here to delete/merge the duplicates faster
      text$+Chr(1)+Str(f) ; Position number, with chr(1) before
      
      ReDim total_number_of_original_synonyms_array$(total_number_of_processed_synonyms)
      total_number_of_original_synonyms_array$(total_number_of_processed_synonyms)=t$ 
      
      ReDim total_number_of_processed_synonyms_array$(total_number_of_processed_synonyms)
      total_number_of_processed_synonyms_array$(total_number_of_processed_synonyms)=text$ 
      total_number_of_processed_synonyms+1
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting..."+FormatNumber(total_number_of_processed_synonyms,0,".",number_separator_chr$)+" <ESC> to abort.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    
    ; Sort the array    
    SortArray(total_number_of_processed_synonyms_array$(),#PB_Sort_Ascending)    
    
    
    ; Step 2 of 2 (comparing synonyms)  
    
    ; First open a hidden window
    If OpenWindow(#WINDOW_SHOW_DUPLICATES,0,0,400+20-5+300-40-30-10-10,-2+9-100+200-50-20-20+100+100+200-30-30-30+10-20-10,"Show/Merge Duplicates",#PB_Window_Invisible|#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    ListIconGadget(#LISTICON_WINDOW_SHOW_DUPLICATES,5,5,300-40,200+100-50+10,"Synonym",100+100-40-5,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_DUPLICATES,1,"Position",100)
    SetGadgetFont(#LISTICON_WINDOW_SHOW_DUPLICATES,FontID(#font_for_language_id))
    TextGadget(#EDITOR_GADGET_SHOW_DUPLICATES_TEXT,5+300+10-40,5,100,default_gadget_height_text,"Preview")
    EditorGadget(#EDITOR_GADGET_SHOW_DUPLICATES,5+300+10-40,-1+5+23,300+40,200+100-50+10-23+1,#PB_Editor_ReadOnly)
    SetGadgetFont(#EDITOR_GADGET_SHOW_DUPLICATES,FontID(#font_for_language_id))
    ButtonGadget(#BUTTON_WINDOW_SHOW_DISPLAY,10,200+100-50+10+10,90,default_gadget_height_button,"Display")
    ButtonGadget(#BUTTON_WINDOW_SHOW_MERGE,5+50+50+50+0+50+100,200+100-50+10+10,100-4-4,default_gadget_height_button,"Merge")
    ButtonGadget(#BUTTON_WINDOW_SHOW_EXPORT,5+50+50+50+0+50+100+100,200+100-50+10+10,100,default_gadget_height_button,"Export")
    ButtonGadget(#BUTTON_WINDOW_SHOW_OKAY,5+50+50+50+50+0+100+10+100+100,200+100-50+10+10,100-10-10,default_gadget_height_button,"Close")
    
    ; Progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 2/2: Comparing... <ESC> to abort.")    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_number_of_processed_synonyms)     
        
    
    
    ; Array to have the extra information for preview+export
    Dim total_number_of_processed_synonyms_array_information$(0)

    
    For f=0 To total_number_of_processed_synonyms-1
      text$=total_number_of_processed_synonyms_array$(f)
      text$=Left(text$,FindString(text$,"|")-1)
      occurrences=0
      occurrences_position=-1
      
      For ff=f To total_number_of_processed_synonyms-1            
                        
        If text$=Left(total_number_of_processed_synonyms_array$(ff) ,FindString(total_number_of_processed_synonyms_array$(ff) ,"|")-1)
          occurrences+1 
          If occurrences_position=-1 : occurrences_position=ff : EndIf
        Else
          Break
        EndIf
                             

      
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 2/2: Comparing..."+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(total_number_of_processed_synonyms,0,".",number_separator_chr$)+" <ESC> to abort.")    
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            RemoveWindowTimer(#WINDOW_MAIN,800)
            CloseWindow(#WINDOW_SHOW_DUPLICATES)
            ProcedureReturn #False
          EndIf
        EndIf      
      
      
      Next ff
      
      f=ff-1      
      
      ; Add duplicate synonyms to listicongadget
      If occurrences>1
        For ff=1 To occurrences
          t$=total_number_of_processed_synonyms_array$(occurrences_position+ff-1)
          AddGadgetItem(#LISTICON_WINDOW_SHOW_DUPLICATES,-1,Left(t$,FindString(t$,#LF$)-1)+#LF$+Str(Val(Mid(t$,FindString(t$,Chr(1))+1,Len(t$)-(FindString(t$,Chr(1)))))+1))
          
          t=CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)
          ReDim total_number_of_processed_synonyms_array_information$(t-1)        
          total_number_of_processed_synonyms_array_information$(t-1)=Mid(t$,FindString(t$,#LF$)+1,FindString(t$,Chr(1))-(FindString(t$,#LF$)+1))
          
        Next ff
      EndIf
      
    Next f    
    
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      

    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
   
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
    
    
    ; No duplicates found, exit the Procedure - 7/AUG/2015
    If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)=0
      MessageRequester("Warning!","No duplicates found.",#PB_MessageRequester_Warning)
      DisableWindow(#WINDOW_MAIN,#False)
      CloseWindow(#WINDOW_SHOW_DUPLICATES)                
      ProcedureReturn
    EndIf
    

    HideWindow(#WINDOW_SHOW_DUPLICATES,#False)
    DisableWindow(#WINDOW_MAIN,#True)
    SetActiveWindow(#WINDOW_SHOW_DUPLICATES)
           
    ; Add the number of words found
    TextGadget(#TEXT_WINDOW_SHOW_DUPLICATES_FOUND,5+5+100+5,200+100-50+10+10,200-20,default_gadget_height_text,"Found:"+Str(CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)))
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_SHOW_DUPLICATES,#PB_Shortcut_Escape,1000)
    
    SetGadgetText(#EDITOR_GADGET_SHOW_DUPLICATES,"Nothing to display")
    
    cancel=#False
    Repeat
      
      
      ; Disable/Enable the "Display" button according to selected/unselected items in the ListIconGadget
      disable_button_display=#False
      If GetGadgetState(#LISTICON_WINDOW_SHOW_DUPLICATES)=-1 : disable_button_display=#True : EndIf
      DisableGadget(#BUTTON_WINDOW_SHOW_DISPLAY,disable_button_display)
      
      
      event=WaitWindowEvent()
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf
      
      
      ; Check if double-clicked with the left mouse button in an item
      If EventGadget()=#LISTICON_WINDOW_SHOW_DUPLICATES And EventType()=#PB_EventType_LeftDoubleClick
        t=GetGadgetState(#LISTICON_WINDOW_SHOW_DUPLICATES)
        If t<>-1
          ;           SetGadgetState(#PANEL_BOX_SYNONYMS,Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,t,1))-1)                    
          selected_thesaurus_word=Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,t,1))-1
          SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,selected_thesaurus_word)     
          global_update_virtual_vertical_scroll_bar(-1)           
        EndIf
      EndIf
      
      
      Select event 
        Case #PB_Event_Gadget
         Select EventGadget()
             
             
           Case #LISTICON_WINDOW_SHOW_DUPLICATES
;              Debug EventType()
             If EventType()=#PB_EventType_LeftClick Or EventType()=768
               t$="Nothing to display"
               t=GetGadgetState(#LISTICON_WINDOW_SHOW_DUPLICATES)
               If t<>-1
                 t$=total_number_of_processed_synonyms_array_information$(t)+#CRLF$
               EndIf
               If GetGadgetText(#EDITOR_GADGET_SHOW_DUPLICATES)<>t$
                 SetGadgetText(#EDITOR_GADGET_SHOW_DUPLICATES,t$)
               EndIf
             EndIf
             
             
           ; Display button
           Case #BUTTON_WINDOW_SHOW_DISPLAY  
              t=GetGadgetState(#LISTICON_WINDOW_SHOW_DUPLICATES)
              If t<>-1
;                 SetGadgetState(#PANEL_BOX_SYNONYMS,Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,t,1))-1)          
                selected_thesaurus_word=Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,t,1))-1
                SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,selected_thesaurus_word)
                global_update_virtual_vertical_scroll_bar(-1)                                              
              EndIf
                          
             
           Case #BUTTON_WINDOW_SHOW_OKAY
             cancel=#True
             
             
           Case #BUTTON_WINDOW_SHOW_MERGE
             ; First it deletes duplicates, from top to bottom, to not miss any
             ; It uses the total_number_of_original_synonyms_array$() array for extra speed
             
             ; Progress bar
             number_of_items_in_listicongadget=CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)
             StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing duplicates... 0/"+Str(number_of_items_in_listicongadget))
             StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_items_in_listicongadget)             
             
             AddWindowTimer(#WINDOW_MAIN,800,800)
             For f=CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)-1 To 1 Step -1
               If GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f,0)=GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,0)
                 If total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f,1))-1)=total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,1))-1)
                   total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f,1))-1)=""
                   RemoveGadgetItem(#LISTICON_WINDOW_SHOW_DUPLICATES,f)
                 EndIf
               EndIf
               
               ; Fix to window stop responding if we click on it while deleting duplicates
               event = WindowEvent()
               If event=#PB_Event_Timer And EventTimer()=800
                 StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,number_of_items_in_listicongadget-f,#PB_StatusBar_Raised)
                 StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing duplicates... "+Str(number_of_items_in_listicongadget-f)+"/"+Str(number_of_items_in_listicongadget))
               EndIf               
               
             Next f
             RemoveWindowTimer(#WINDOW_MAIN,800)
             
             
             
             ; Second, it merges the synonyms by comparing each meaning in each synonym
             ; It uses the total_number_of_original_synonyms_array$() array for extra speed
             
             ; Progress bar
             number_of_items_in_listicongadget=CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)
             StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Merging duplicates... 0/"+FormatNumber(number_of_items_in_listicongadget,0,".",number_separator_chr$))
             StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_items_in_listicongadget)             
             
             ; Only works if number of items in listicongadget is > 1
             If number_of_items_in_listicongadget>1
               AddWindowTimer(#WINDOW_MAIN,800,800)
               For f=number_of_items_in_listicongadget-1 To 1 Step -1
                 
                 text_current$=GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f,0)
                 text_current_minus1$=GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,0)                 
                 t_current=Val(Mid(text_current$,FindString(text_current$,"|")+1))
                 t_current_minus1=Val(Mid(text_current_minus1$,FindString(text_current_minus1$,"|")+1))

                   
                 ; Change the two variables above into a real line - To fix the bug of not working
                 text_current$=total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f,1))-1)
                 text_current_minus1$=total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,1))-1)                   
                 
                 
                 ; Convert the synonyms into an array that will contain each synonym
                 
                 counter_new=0
                 a$=""
If Left(text_current$,FindString(text_current$,"|"))=Left(text_current_minus1$,FindString(text_current_minus1$,"|"))               
                 
                 ; First, the current F
                 Dim array_with_synonym_current$(t_current)
                 position=FindString(text_current$,#LF$)+1
                 For ff=1 To t_current
                   a$=""
                   For fff=position To Len(text_current$)
                     If Mid(text_current$,fff,1)<>#LF$
                       a$+Mid(text_current$,fff,1)
                     Else
                       Break
                     EndIf
                   Next fff
                   position=fff+1
                   array_with_synonym_current$(ff)=a$
                 Next ff                   
                   
                   
                 ; Second, the current F-1
                 Dim array_with_synonym_current_minus1$(t_current_minus1)
                 position=FindString(text_current_minus1$,#LF$)+1
                 For ff=1 To t_current_minus1
                   a$=""
                   For fff=position To Len(text_current_minus1$)
                     If Mid(text_current_minus1$,fff,1)<>#LF$
                       a$+Mid(text_current_minus1$,fff,1)
                     Else
                       Break
                     EndIf
                   Next fff
                   position=fff+1
                   array_with_synonym_current_minus1$(ff)=a$
                 Next ff                   
                   
                   
                 ; Now attempt to match both arrays
                 a$=""
                 counter_new=0
                 For ff=1 To t_current
  
                   flag=#False ; Means no match was found
                     
                   For fff=1 To t_current_minus1

                     If array_with_synonym_current_minus1$(fff)=array_with_synonym_current$(ff)
                       flag=#True ; Found a match
                       Break
                     EndIf
                       
                   Next fff
                     
                   ; No match found, so it inserts it in the F-1 (variable a$)
                   If flag=#False
                     counter_new+1
                     a$+#LF$+array_with_synonym_current$(ff)
                   EndIf
                     
                 Next ff
                   
; EndIf                 


; Debug "F:"+Str(f)+" counter_new:"+Str(counter_new)+" t_current:"+Str(t_current)+" t_current_minus1:"+Str(t_current_minus1)
Debug "counter new:"+Str(counter_new)

                 ; Add new meaning inside a synonym
                 If counter_new>0

                    ; First it checks if the number of synonyms isn't above the maximum
                    If t_current_minus1+counter_new>#max_number_meanings

                      MessageRequester("Warning!","Found synonym with more than "+Str(#max_number_meanings)+" meanings.",#PB_MessageRequester_Warning)
                      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
                      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)  
                        
                      ; Close the Duplicates window and activate the main window
                      DisableWindow(#WINDOW_MAIN,#False)
                      CloseWindow(#WINDOW_SHOW_DUPLICATES)                            
                      
                      ; Exit the procedure
                      ProcedureReturn #False
                      
                    EndIf
                 EndIf      
                  
                  
                    ; Now it updates the number of meanings in the synonym in the array that looks like the original                   
                    ; aaa bbb   |   100  <chr10> zzz  <chr10>  yyy
                    ;  1   2    3   456    7      8      9      10
                    old_synonym$=GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,0)
                    new_synonym$=Left(old_synonym$,FindString(old_synonym$,"|"))+Str(t_current_minus1+counter_new)                   
; Debug "new_synonym$:"+new_synonym$
                    ; Don't forget to replace in the listicongadget the F-1 with the updated number of meanings
                    SetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,new_synonym$,0)
                    
                    
;                  EndIf   
                 
; Debug "New Line:"+new_synonym$+Mid(total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,1))-1),Len(old_synonym$)+1)+a$                 
                 ; Change in the original synonym array which will populate the original list in the end
                 total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,1))-1)=new_synonym$+Mid(total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,1))-1),Len(old_synonym$)+1)+a$
                                  
                 
                 ; Removes the synonym line
                  total_number_of_original_synonyms_array$(Val(GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f,1))-1)=""
                  
                  
EndIf                  
                 
                 ; Removes from the listicongadget used in the show duplicates
                 RemoveGadgetItem(#LISTICON_WINDOW_SHOW_DUPLICATES,f)

               
                 ; Fix to window stop responding if we click on it while deleting duplicates
                 event = WindowEvent()
                 If event=#PB_Event_Timer And EventTimer()=800
                   StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,number_of_items_in_listicongadget-f,#PB_StatusBar_Raised)
                   StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Merging duplicates... "+FormatNumber(number_of_items_in_listicongadget-f,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_items_in_listicongadget,0,".",number_separator_chr$))
                 EndIf               
               
               Next f
               RemoveWindowTimer(#WINDOW_MAIN,800) 
               update_number_synonyms_selected_none() ; 25/DEC/2016
             EndIf
             ClearGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES) ; 14/AUG/2015: If one item is there, it gets removed, so that we don't see it while populating the main listicongadget
             
             
             
             ; Populate the listicongadget *START*
             ; Sort the original array and populate the listicongadget with it
    
    
             ; Sort the array and delete the list 
             SortArray(total_number_of_original_synonyms_array$(),#PB_Sort_Ascending|#PB_Sort_NoCase)
             ClearGadgetItems(#PANEL_BOX_SYNONYMS)
    
             ; Progress bar
             StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating... 0/"+FormatNumber(number_of_synonyms,0,".",number_separator_chr$))
             StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_synonyms)


             ; copy from array back to list
             AddWindowTimer(#WINDOW_MAIN,800,800)
             time=ElapsedMilliseconds()
             new_number_of_synonyms=0 ; To have the deleted ones into account
             For f=0 To number_of_synonyms-1
                t$=total_number_of_original_synonyms_array$(f)
                If t$<>""
                  synonyms$(new_number_of_synonyms)=t$
                  new_number_of_synonyms+1
                EndIf
             
      
                ; Fix to window stop responding if we click on it while adding synonyms
                event = WindowEvent()
                If Event = #PB_Event_Timer And EventTimer() = 800 : EndIf     
      
                If ElapsedMilliseconds()>time+800
                    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating..."+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_synonyms,0,".",number_separator_chr$))            
                    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0+f,#PB_StatusBar_Raised,0,number_of_synonyms)
                    time=ElapsedMilliseconds() 
                EndIf    
             Next f    
             RemoveWindowTimer(#WINDOW_MAIN,800)

             
             ; Reset status bar and display "OK." in it
             StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
             StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
             number_of_synonyms=new_number_of_synonyms
             ReDim synonyms$(new_number_of_synonyms-1)             
             ReDim synonyms_selected(new_number_of_synonyms-1)                
             update_number_synonyms_selected_none() ; 25/DEC/2016
             
             
             ; Refresh the ListIconGadget
             selected_thesaurus_word=-1
             SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
             global_update_virtual_vertical_scroll_bar(-1)              
             
             
             ; Exit the procedure
             thesaurus_changed=#True ;Thesaurus changed
             cancel=#True
             
             ; Populate the listicongadget *END*             
             
             
             
           Case #BUTTON_WINDOW_SHOW_EXPORT
             If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)>0
               flag=#False
               file$=SaveFileRequester("Export Duplicates to file","","Text Documents (*.txt)|*.txt",0)
               If file$<>""
                 If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
                 
                 t=ReadFile(1,file$)
                 If t<>0
                   CloseFile(1)
                   If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                       DeleteFile(file$) ; Delete .TXT file                 
                   Else
                       flag=#True ; Abort the save as the file already exists and pressed to not overwrite it
                   EndIf  
                 EndIf    
                 
                 If flag=#False
                    ; Saving the .TXT file
                    AddWindowTimer(#WINDOW_MAIN,800,800)
                    OpenFile(1,file$)      
                    WriteStringFormat(1,#PB_UTF8)
                    
                    WriteString(1,"DUPLICATES FOUND:"+FormatNumber(CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES),0,".",number_separator_chr$)+#LF$,#PB_UTF8)
                    WriteString(1,""+#LF$,#PB_UTF8)

                    For f=1 To CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)
;                       t$=ReplaceString(total_number_of_processed_synonyms_array_information$(f-1),#LF$,#CRLF$) ; Removed on 21/FEB/2020
                      t$=total_number_of_processed_synonyms_array_information$(f-1) ; Added on 21/FEB/2020
                      WriteString(1,GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,0)+#LF$+t$+#LF$+#LF$,#PB_UTF8)
                      
                      ; Fix to window stop responding if we click on it while exporting
                      event = WindowEvent()
                      If event=#PB_Event_Timer And EventTimer()=800 : EndIf 
                
                    Next f
                    CloseFile(1)
                    RemoveWindowTimer(#WINDOW_MAIN,800)          
                 EndIf
                 
               EndIf
               
             EndIf
         EndSelect
         
      EndSelect   
      
    Until cancel=#True
    
    ; Close the Duplicates window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_SHOW_DUPLICATES)    
    
EndProcedure




; To combine the synonyms
; BASIC VERSION: doesn't accept -| ( ) and |
; More to help Shantanu Oak (one synonym per line)
;
; x|3
; a
; b
; c
; BECOMES:
; a|3
; x
; b
; c
;
; b|3
; a
; x
; c
;
; c|3
; a
; b
; x
;
; V1.0 -  7/AUG/2015
; V1.1 - 11/AUG/2015
;         o Removed the merging code since it will be done in other procedure.
;         o Improved the code
; V1.2 - 12/FEB/2016
;        o Thesaurus now uses dynamic arrays
; V1.3 - 16/FEB/2016
;        o Now in the combine synonyms it says how much are new
; V1.4 - 25/DEC/2016
;        o Started implementing the multiselection.
; V1.5 - 8/JUN/2017
;        o undo/redo support  
; V1.6 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure combine_synonyms()
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Combine. 1/3: Extracting... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
      ProcedureReturn #False
    EndIf 
      
    synonym_store_undo(#True) ; 8/JUN/2017
    
    ; Step 1 of 3 (extracting synonyms from listicongadget to array)
    AddWindowTimer(#WINDOW_MAIN,800,800)
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_synonyms)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    total_number_of_processed_synonyms=0
    Dim total_number_of_processed_synonyms_array$(0)
    
    
    original_number_of_synonyms=number_of_synonyms ; so that at the end it says in how much it increased
    
    For f=0 To number_of_synonyms-1
      
      ; Get a line of synonyms
      text$=synonyms$(f)
               
      ; Check if synonyms are not BASIC [[BASIC VERSION doesn't accept -| ( ) and | ]]
      If CountString(text$,"|")>1 Or FindString(text$,"-|")>0 Or FindString(text$,"(")>0 Or FindString(text$,")")>0
        MessageRequester("Warning!","Basic combine doesn't accept synonyms with:"+#LF$+"-| ( ) and |"+#LF$+#LF$+"It only accepts one synonym per line.",#PB_MessageRequester_Warning)
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        RemoveWindowTimer(#WINDOW_MAIN,800)
        ProcedureReturn #False       
      EndIf
      
      
      ReDim total_number_of_processed_synonyms_array$(total_number_of_processed_synonyms)
      total_number_of_processed_synonyms_array$(total_number_of_processed_synonyms)=text$      
      total_number_of_processed_synonyms+1
                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Combine. 1/3: Extracting..."+Str(f)+"/"+Str(number_of_synonyms)+" <ESC> To abort.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    
    ; Sort the array    
    SortArray(total_number_of_processed_synonyms_array$(),#PB_Sort_Ascending)    
    
    
    
    ; Step 2 of 3 (combining synonyms)    
        
    ; Progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Combine. 2/3: Combining... <ESC> to abort.")    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_number_of_processed_synonyms)     
            
    total_number_of_combined_synonyms=total_number_of_processed_synonyms
    
    For f=0 To total_number_of_processed_synonyms-1
      
      
      ; Get a line of synonyms + number of synonyms in the line
      text$=total_number_of_processed_synonyms_array$(f)                
      t=Val(Mid(text$,FindString(text$,"|")+1,FindString(text$,#LF$)-FindString(text$,"|")-1))
      
      
      ; Convert the synonyms into an array that will contain each synonym
      ; array(0)=original synonym without the "|"
      Dim array_with_synonym$(t)
      array_with_synonym$(0)=Left(text$,FindString(text$,"|")-1)
      position=FindString(text$,#LF$)+1
      For ff=1 To t
        a$=""
        For fff=position To Len(text$)
          If Mid(text$,fff,1)<>#LF$
            a$+Mid(text$,fff,1)
          Else
            Break
          EndIf
        Next fff
        position+Len(a$)+1
        array_with_synonym$(ff)=a$
;         Debug "a$:"+a$
      Next ff
      
      
      ; Combine
      For ff=1 To t        
        text$=""        
        For fff=1 To t
          If fff=ff
            text$=array_with_synonym$(fff)+"|"+Str(t)+text$+#LF$+array_with_synonym$(0)
          Else
            text$+#LF$+array_with_synonym$(fff)
          EndIf
        Next fff
; Debug "Text:"+text$        
        
        ; Copy the combination to the array that stores all the temporary synonyms
        ReDim total_number_of_processed_synonyms_array$(total_number_of_combined_synonyms)
        total_number_of_processed_synonyms_array$(total_number_of_combined_synonyms)=text$      
        total_number_of_combined_synonyms+1
                
      Next ff


      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
         StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Combine. 2/3: Combining..."+Str(f)+"/"+Str(total_number_of_processed_synonyms)+" <ESC> to abort.")    
         StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
        EndIf
      EndIf      
      
            
    Next f    
    
    
    
    ; Step 3 of 3 (populating synonyms which means copying them back to the listicongadget)

    
    ; Sort the temporary array and populate the listicongadget with it
    
    
    ; Sort the array and delete the list 
    SortArray(total_number_of_processed_synonyms_array$(),#PB_Sort_Ascending|#PB_Sort_NoCase)
    ClearGadgetItems(#PANEL_BOX_SYNONYMS)
    
    ; Progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Combine. 3/3: Populating... 0/"+Str(total_number_of_combined_synonyms))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_number_of_combined_synonyms)

    
    ; Redim the array that stores the synonyms
    ReDim synonyms$(total_number_of_combined_synonyms-1)
    ReDim synonyms_selected(total_number_of_combined_synonyms-1)
    update_number_synonyms_selected_none()
    
    
    ; copy from array back to list
    AddWindowTimer(#WINDOW_MAIN,800,800)
    time=ElapsedMilliseconds()
    For f=0 To total_number_of_combined_synonyms-1
      t$=total_number_of_processed_synonyms_array$(f)
      synonyms$(f)=t$
      
      ; Fix to window stop responding if we click on it while adding synonyms
      event = WindowEvent()
      If Event = #PB_Event_Timer And EventTimer() = 800 : EndIf     
      
      If ElapsedMilliseconds()>time+800
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Combine. 3/3: Populating..."+Str(f)+"/"+Str(total_number_of_combined_synonyms))            
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0+f,#PB_StatusBar_Raised,0,total_number_of_combined_synonyms)
        time=ElapsedMilliseconds() 
      EndIf
      
    Next f    
    RemoveWindowTimer(#WINDOW_MAIN,800)
    
    
    ; Reset status bar and display "OK." in it
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "New synonyms:"+Str(total_number_of_combined_synonyms-original_number_of_synonyms))
    number_of_synonyms=total_number_of_combined_synonyms
    thesaurus_changed=#True ;Thesaurus changed
    
    ; Refresh the ListIconGadget
    SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
    global_update_virtual_vertical_scroll_bar(-1) 
        
    
EndProcedure




; To show the total of items selected
; 
; V1.0 - 24/DEC/2016
; 
Procedure update_number_synonyms_selected()
  
  t$="Selected:"
  If number_of_synonyms_total_selected=0 : t$+"None" : EndIf
  If number_of_synonyms_total_selected<>0 And number_of_synonyms_total_selected<>number_of_synonyms : t$+Str(number_of_synonyms_total_selected) : EndIf
  If number_of_synonyms_total_selected=number_of_synonyms And number_of_synonyms<>0 : t$+"All" : EndIf
  
  If GetGadgetText(#TEXT_PANEL_BOX_SYNONYMS_NUMBER_WORDS_SELECTED)<>t$ : SetGadgetText(#TEXT_PANEL_BOX_SYNONYMS_NUMBER_WORDS_SELECTED,t$) : EndIf
    
EndProcedure




; To select ALL in the array
; 
; V1.0 - 24/DEC/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.  
; 
Procedure update_number_synonyms_selected_all()
  
  If number_of_synonyms=0
    MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf
  
  If number_of_synonyms>0
    For f=0 To number_of_synonyms-1
      synonyms_selected(f)=#True
    Next f
  EndIf
  
  number_of_synonyms_total_selected=number_of_synonyms
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; To select NONE in the array
; 
; V1.0 - 24/DEC/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign. 
; 
Procedure update_number_synonyms_selected_none()
  
  If number_of_synonyms=0
    MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  If number_of_synonyms>0
    For f=0 To number_of_synonyms-1
      synonyms_selected(f)=#False
    Next f
  EndIf  
  
  number_of_synonyms_total_selected=0
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; To INVERT in the array
; 
; V1.0 - 2/MAR/2018
;   
Procedure update_number_synonyms_selected_invert()
  
  If number_of_synonyms=0
    MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  count=0
  If number_of_synonyms>0
    For f=0 To number_of_synonyms-1
      v=synonyms_selected(f)
      If v=#False
        v=#True
        count+1
      Else
        v=#False
      EndIf            
      synonyms_selected(f)=v
    Next f
  EndIf  
  
  number_of_synonyms_total_selected=count
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; Count the number of selected synonyms after a delete and such
;
; V1.0 - 24/DEC/2016
; 
Procedure update_number_synonyms_selected_count()
  
  number_of_synonyms_total_selected=0
  
  If number_of_synonyms>0
    For f=0 To number_of_synonyms-1
      If synonyms_selected(f)=#True : number_of_synonyms_total_selected+1 : EndIf
    Next f
  EndIf    
  
EndProcedure




; Delete the number of selected synonyms
;
; V1.0 - 24/DEC/2016
; V1.1 - 1/JAN/2017
;        o Fix deleting selected only when there was one item
; V1.2 - 8/JUN/2017
;        o undo/redo support 
; V1.3 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure update_number_synonyms_selected_delete()
  
  
  If number_of_synonyms=0
    MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf
  
  
  If number_of_synonyms_total_selected=0
    MessageRequester("Warning!","No selected synonyms found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  
  t$=""
  If number_of_synonyms_total_selected>1 : t$="s" : EndIf
  If MessageRequester("Warning!","Really delete "+Str(number_of_synonyms_total_selected)+" synonym"+t$+"?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No  
    ProcedureReturn
  EndIf
  
  synonym_store_undo(#False) ; 8/JUN/2017
  
  Dim buffer$(number_of_synonyms-1)
  
  counter=0
  position=0
  For f=0 To number_of_synonyms-1
    If synonyms_selected(f)=#False
      buffer$(position)=synonyms$(f)
      position+1
    EndIf
    If synonyms_selected(f)=#True
      counter+1
      synonyms_selected(f)=#False
    EndIf
  Next f
  
  CopyArray(buffer$(),synonyms$())
  number_of_synonyms-counter  
  zero_items=#False
  If number_of_synonyms=0
    zero_items=#True
    number_of_synonyms=1
  EndIf
  ReDim synonyms$(number_of_synonyms-1)
  ReDim synonyms_selected(number_of_synonyms-1)
  
  number_of_synonyms_total_selected=0
  
  SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
  selected_thesaurus_word=-1 ; Reset the current synonym selected
  
  ; Fix deleting only when there was one item
  If zero_items=#True : number_of_synonyms=0 : EndIf
  
  thesaurus_changed=#True
  update_number_synonyms()
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; Clone the synonyms
;
; V1.0 - 18/MAR/2017
; V1.1 - 8/JUN/2017
;        o undo/redo support  
; V1.2 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure clone_synonym()
        
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    
    t=selected_thesaurus_word
    If t=-1
      MessageRequester("Warning!","No synonym selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf        
    
    synonym_store_undo(#False) ; 8/JUN/2017
    
    thesaurus_changed=#True
    ReDim synonyms$(number_of_synonyms)      
    ReDim synonyms_selected(number_of_synonyms)   
    synonyms$(number_of_synonyms)=synonyms$(t)
    number_of_synonyms+1
    SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
    update_number_synonyms()
    global_update_virtual_vertical_scroll_bar(-1)
    update_vertical_scroll_bar_thesaurus_current_item()  
        
  
EndProcedure




; Disable undo + redo
;
; V1.0 - 8/JUN/2017
;
Procedure synonym_disable_undo_redo()
  
  DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_UNDO,#True)
  Dim synonym_undo_array(0)
  Dim synonym_undo_array_total_selected(0)    
  synonym_undo=#False
  synonym_undo_number_items=0
  synonym_undo_selected_item=-1
  synonym_undo_items_total_selected=0
  synonym_undo_top_of_listicongadget=0
  
  
  DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_REDO,#True)    
  Dim synonym_redo_array(0)
  Dim synonym_redo_array_total_selected(0)
  synonym_redo=#False
  synonym_redo_number_items=0
  synonym_redo_selected_item=-1
  synonym_redo_items_total_selected=0
  synonym_redo_top_of_listicongadget=0  
  
EndProcedure




; Store undo
;
; V1.0 - 08/JUN/2017
; V1.1 - 23/MAR/2022
;        o Disable main GUI for input/output operations
;
Procedure synonym_store_undo(disable_gadgets)
  Debug "STORE UNDO"
  
  ; Enable undo button and disable redo button
  If disable_gadgets=#False
    DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_UNDO,#False)
    DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_REDO,#True)  
  EndIf 
  
  synonym_undo=#True
  synonym_redo=#False
  
  Dim synonym_undo_array$(ArraySize(synonyms$()))
  CopyArray(synonyms$(),synonym_undo_array$())
  Dim synonym_undo_array_total_selected(ArraySize(synonyms_selected()))  
  CopyArray(synonyms_selected(),synonym_undo_array_total_selected())
  synonym_undo_number_items=number_of_synonyms
  synonym_undo_selected_item=selected_thesaurus_word
  synonym_undo_items_total_selected=number_of_synonyms_total_selected
  synonym_undo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)
  
EndProcedure




; Restore undo
;
; V1.0 - 8/JUN/2017
;
Procedure synonym_restore_undo()
  Debug "RESTORE UNDO"
  
  ; Disable undo button and enable redo button
  DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_UNDO,#True)
  DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_REDO,#False)    
  
  synonym_undo=#False
  synonym_redo=#True
  
  Dim synonyms$(ArraySize(synonym_undo_array$()))
  CopyArray(synonym_undo_array$(),synonyms$())  
  Dim synonyms_selected(ArraySize(synonym_undo_array_total_selected()))
  CopyArray(synonym_undo_array_total_selected(),synonyms_selected())
  number_of_synonyms=synonym_undo_number_items
  selected_thesaurus_word=synonym_undo_selected_item
  number_of_synonyms_total_selected=synonym_undo_items_total_selected  
;   synonym_undo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)
  
  
  ; Count number of selected words
  update_number_synonyms_selected_count()
      
      
  ; Update number of selected words
  update_number_synonyms_selected()
      
      
  ; Update number of words
  update_number_synonyms()
      
      
  ; Reset thesaurus
  selected_thesaurus_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
  global_update_virtual_vertical_scroll_bar(-1)
  
  selected_thesaurus_word=synonym_undo_selected_item           
      
  ; Place back the first item on top of the ListIconGadget
  SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,synonym_undo_top_of_listicongadget)
  global_update_virtual_vertical_scroll_bar(-1)
 
  
EndProcedure




; Store redo
;
; V1.0 - 8/JUN/2017
;
Procedure synonym_store_redo()
  Debug "STORE REDO"
  
  ; Enable redo button and disable undo button
  DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_UNDO,#True)
  DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_REDO,#False)  
  
  synonym_undo=#False
  synonym_redo=#True
  
  Dim synonym_redo_array$(ArraySize(synonyms$()))
  CopyArray(synonyms$(),synonym_redo_array$())
  Dim synonym_redo_array_total_selected(ArraySize(synonyms_selected()))  
  CopyArray(synonyms_selected(),synonym_redo_array_total_selected())
  synonym_redo_number_items=number_of_synonyms
  synonym_redo_selected_item=selected_thesaurus_word
  synonym_redo_items_total_selected=number_of_synonyms_total_selected
  synonym_redo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)  
  
EndProcedure




; Restore redo
;
; V1.0 - 8/JUN/2017
;
Procedure synonym_restore_redo()
  Debug "RESTORE REDO"
  
  ; Disable redo button and enable undo button
  DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_UNDO,#False) 
  DisableGadget(#BUTTON_PANEL_BOX_SYNONYMS_REDO,#True)     
  
  synonym_undo=#True
  synonym_redo=#False
  
  Dim synonyms$(ArraySize(synonym_redo_array$()))
  CopyArray(synonym_redo_array$(),synonyms$())  
  Dim synonyms_selected(ArraySize(synonym_redo_array_total_selected()))
  CopyArray(synonym_redo_array_total_selected(),synonyms_selected())
  number_of_synonyms=synonym_redo_number_items
  selected_thesaurus_word=synonym_redo_selected_item
  number_of_synonyms_total_selected=synonym_redo_items_total_selected  
;   synonym_redo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR)
  
  
  ; Count number of selected words
  update_number_synonyms_selected_count()
      
      
  ; Update number of selected words
  update_number_synonyms_selected()
      
      
  ; Update number of words
  update_number_synonyms()
      
      
  ; Reset thesaurus
  selected_thesaurus_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_synonyms-1)
  global_update_virtual_vertical_scroll_bar(-1)
  
  selected_thesaurus_word=synonym_redo_selected_item
      
  ; Place back the first item on top of the ListIconGadget
  SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,synonym_redo_top_of_listicongadget)
  global_update_virtual_vertical_scroll_bar(-1)  
  
EndProcedure


  
  
  ; Sorts the list of synonyms based on number + name
  ;
  ; V1.0 - 17/MAY/2020
  ; V1.1 - 02/DEC/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure sort_on_number_synonyms()
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting Thesaurus on number...")
    
    If number_of_synonyms=0
      MessageRequester("Warning!","No synonyms found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
      ProcedureReturn #False
    EndIf
    
    synonym_store_undo(#True) ; 8/JUN/2017
    
    ; Show the progress bar
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_synonyms*3) 
    
    ; Thesaurus set to changed
    selected_thesaurus_word=-1 ; Reset the current word selected
    thesaurus_changed=1
    update_number_synonyms()
    
    
    time_taken=ElapsedMilliseconds()
    AddWindowTimer(#WINDOW_MAIN,800,800)
    
    
    ; Remove accents from entries
    ReDim sort_arrays_in_latin(number_of_synonyms-1)
    For f=0 To number_of_synonyms-1
      
      t$=StringField(synonyms$(f),1,#LF$)
      t_number$=Right("0000000000"+StringField(t$,2,"|"),10)
      t_name$=RemoveAccentsString(StringField(t$,1,"|"))
      
      sort_arrays_in_latin(f)\in_latin=t_number$+t_name$
      sort_arrays_in_latin(f)\in_ascii=synonyms$(f)
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f+1,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing accents... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f
    
    
    ; Sort the array and delete the list 
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting on number... <ESC> to abort. Found:"+FormatNumber(number_of_synonyms,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    For f=0 To number_of_synonyms-1
      ReplaceString(sort_arrays_in_latin(f)\in_latin,"|",Chr(9),#PB_String_InPlace,1,1)
    Next f    
    SortStructuredArray(sort_arrays_in_latin(),#PB_Sort_Ascending,OffsetOf(sort_arrays_in_latin\in_latin.s),TypeOf(sort_arrays_in_latin\in_latin.s))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,number_of_synonyms*2,#PB_StatusBar_Raised)
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting on number... <ESC> to abort. Found:"+FormatNumber(number_of_synonyms,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
    
    ; Populate the list with the sorted words
    For f=0 To number_of_synonyms-1
      
      synonyms$(f)=sort_arrays_in_latin(f)\in_ascii
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,(number_of_synonyms*2)+f+1,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating with accents... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f    
    
    
    ; Populate the ListIconGadget
    ClearGadgetItems(#PANEL_BOX_SYNONYMS)       
    SetGadgetState(#PANEL_BOX_SYNONYMS_VERTICAL_BAR,0)
    global_update_virtual_vertical_scroll_bar(-1)
    
    
    ; Clear progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,1) ; Reset progress bar - 21/MAR/2014
    
    ; No selected items
    update_number_synonyms_selected_none()
        
    
  EndProcedure
  


; IDE Options = PureBasic 6.04 LTS (Windows - x64)
; CursorPosition = 557
; FirstLine = 542
; Folding = ------
; EnableXP
; CompileSourceDirectory
; EnableUnicode